@@ -86,6 +86,8 @@ public:
         uint8_t data[];
     };
 
+    typedef layout* layout_ptr;
+
     Alternative(std::string name,
                 std::string moduleName,
                 const std::vector<std::pair<std::string, NamedTuple*> >& subtypes,
@@ -158,6 +160,68 @@ public:
         m_subtypes[which(self)].second->serialize(eltPtr(self), buffer, which(self));
     }
 
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        if (m_all_alternatives_empty) {
+            return 0;
+        }
+
+        instance_ptr* p = *(instance_ptr**)instance;
+
+        if (alreadyVisited.find(p) != alreadyVisited.end()) {
+            return 0;
+        }
+
+        alreadyVisited.insert(p);
+
+        if (outSlabs && Slab::slabForAlloc(p)) {
+            outSlabs->insert(Slab::slabForAlloc(p));
+            return 0;
+        }
+
+        return
+            bytesRequiredForAllocation(m_subtypes[which(instance)].second->bytecount() + sizeof(layout)) +
+            m_subtypes[which(instance)].second->deepBytecount(eltPtr(instance), alreadyVisited, outSlabs);
+    }
+
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        if (m_all_alternatives_empty) {
+            ((uint8_t*)dest)[0] = ((uint8_t*)src)[0];
+            return;
+        }
+
+        layout_ptr& destRecordPtr = *(layout**)dest;
+        layout_ptr& srcRecordPtr = *(layout**)src;
+
+        size_t typeIx = srcRecordPtr->which;
+
+        auto it = context.alreadyAllocated.find((instance_ptr)srcRecordPtr);
+
+        if (it == context.alreadyAllocated.end()) {
+            destRecordPtr = (layout*)context.slab->allocate(
+                sizeof(layout) + m_subtypes[typeIx].second->bytecount(),
+                this
+            );
+            destRecordPtr->refcount = 0;
+            destRecordPtr->which = srcRecordPtr->which;
+
+            m_subtypes[typeIx].second->deepcopy(
+                destRecordPtr->data,
+                srcRecordPtr->data,
+                context
+            );
+
+            context.alreadyAllocated[(instance_ptr)srcRecordPtr] = (instance_ptr)destRecordPtr;
+        } else {
+            destRecordPtr = (layout_ptr)it->second;
+        }
+
+        destRecordPtr->refcount++;
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         if (wireType != WireType::SINGLE) {
@@ -178,7 +242,7 @@ public:
             return;
         }
 
-        *(layout**)self = (layout*)malloc(
+        *(layout**)self = (layout*)tp_malloc(
             sizeof(layout) +
             m_subtypes[which].second->bytecount()
             );
