@@ -33,7 +33,11 @@ public:
     }
 
     ShaHash _computeIdentityHash(MutuallyRecursiveTypeGroup* groupHead = nullptr) {
-        return ShaHash(1, m_typeCategory) + m_first_arg->identityHash(groupHead);
+        return (
+            ShaHash(1, m_typeCategory) +
+            m_first_arg->identityHash(groupHead) +
+            ShaHash(m_funcName)
+        );
     }
 
     template<class visitor_type>
@@ -98,6 +102,18 @@ public:
         return m_first_arg->hash(left);
     }
 
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        return m_first_arg->deepBytecount(instance, alreadyVisited, outSlabs);
+    }
+
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        m_first_arg->deepcopy(dest, src, context);
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         m_first_arg->deserialize(self, buffer, wireType);
