@@ -29,6 +29,8 @@ public:
         uint8_t data[];
     };
 
+    typedef layout* layout_ptr;
+
     BytesType() : Type(TypeCategory::catBytes)
     {
         m_name = "bytes";
@@ -48,6 +50,68 @@ public:
         buffer.write_bytes(eltPtr(self, 0), count(self));
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        layout_ptr& destLayout = *(layout**)dest;
+        layout_ptr& srcLayout = *(layout**)src;
+
+        if (!srcLayout) {
+            destLayout = srcLayout;
+            return;
+        }
+
+        if (srcLayout->refcount == 1) {
+            destLayout = (layout_ptr)context.slab->allocate(sizeof(layout) + srcLayout->bytecount, this);
+            destLayout->refcount = 1;
+            destLayout->hash_cache = srcLayout->hash_cache;
+            destLayout->bytecount = srcLayout->bytecount;
+            memcpy(destLayout->data, srcLayout->data, srcLayout->bytecount);
+            return;
+        }
+
+        auto it = context.alreadyAllocated.find((instance_ptr)srcLayout);
+        if (it == context.alreadyAllocated.end()) {
+            destLayout = (layout_ptr)context.slab->allocate(sizeof(layout) + srcLayout->bytecount, this);
+            destLayout->refcount = 0;
+            destLayout->hash_cache = srcLayout->hash_cache;
+            destLayout->bytecount = srcLayout->bytecount;
+            memcpy(destLayout->data, srcLayout->data, srcLayout->bytecount);
+
+            context.alreadyAllocated[(instance_ptr)srcLayout] = (instance_ptr)destLayout;
+        } else {
+            destLayout = (layout_ptr)it->second;
+        }
+
+        destLayout->refcount++;
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        layout* l = *(layout**)instance;
+
+        if (!l) {
+            return 0;
+        }
+
+        if (l->refcount != 1) {
+            if (alreadyVisited.find((void*)l) != alreadyVisited.end()) {
+                return 0;
+            }
+
+            alreadyVisited.insert((void*)l);
+        }
+
+        if (outSlabs && Slab::slabForAlloc(l)) {
+            outSlabs->insert(Slab::slabForAlloc(l));
+            return 0;
+        }
+
+
+        return bytesRequiredForAllocation(l->bytecount + sizeof(layout));
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         if (wireType != WireType::BYTES) {
