@@ -43,6 +43,8 @@ public:
         unsigned char data[];
     };
 
+    typedef layout* layout_ptr;
+
     Class(std::string name, HeldClass* inClass) :
             Type(catClass),
             m_heldClass(inClass)
@@ -120,7 +122,7 @@ public:
             std::string inName,
             const std::vector<Class*>& bases,
             bool isFinal,
-            const std::vector<std::tuple<std::string, Type*, Instance> >& members,
+            const std::vector<MemberDefinition>& members,
             const std::map<std::string, Function*>& memberFunctions,
             const std::map<std::string, Function*>& staticFunctions,
             const std::map<std::string, Function*>& propertyFunctions,
@@ -177,6 +179,8 @@ public:
 
     void setAttribute(instance_ptr self, int64_t ix, instance_ptr elt) const;
 
+    void delAttribute(instance_ptr self, int64_t ix) const;
+
     bool checkInitializationFlag(instance_ptr self, int64_t ix) const;
 
     ShaHash _computeIdentityHash(MutuallyRecursiveTypeGroup* groupHead = nullptr) {
@@ -186,6 +190,68 @@ public:
     bool cmp(instance_ptr left, instance_ptr right, int pyComparisonOp, bool suppressExceptions);
     static bool cmpStatic(Class* T, instance_ptr left, instance_ptr right, int64_t pyComparisonOp);
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        //layout_ptr& destRecordPtr = *(layout**)dest;
+        layout_ptr srcRecordPtr = instanceToLayout(src);
+
+        auto it = context.alreadyAllocated.find((instance_ptr)srcRecordPtr);
+
+        if (it == context.alreadyAllocated.end()) {
+            // we could have a pointer to a subclass of 'this', in which case
+            // the layout could be larger and have more fields than
+            // mHeldClass would indicate.
+            HeldClass* actualHeldClassType = srcRecordPtr->vtable->mType;
+
+            layout_ptr destRecordPtr = (layout_ptr)context.slab->allocate(
+                sizeof(layout) + actualHeldClassType->bytecount(),
+                this
+            );
+
+            destRecordPtr->refcount = 0;
+            destRecordPtr->vtable = srcRecordPtr->vtable;
+
+            actualHeldClassType->deepcopy(
+                destRecordPtr->data,
+                srcRecordPtr->data,
+                context
+            );
+
+            context.alreadyAllocated[(instance_ptr)srcRecordPtr] = (instance_ptr)destRecordPtr;
+        }
+
+        ((layout_ptr)context.alreadyAllocated[(instance_ptr)srcRecordPtr])->refcount++;
+
+        initializeInstance(
+            dest,
+            (layout_ptr)context.alreadyAllocated[(instance_ptr)srcRecordPtr],
+            instanceToDispatchTableIndex(src)
+        );
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        layout_ptr p = instanceToLayout(instance);
+
+        if (alreadyVisited.find((void*)p) != alreadyVisited.end()) {
+            return 0;
+        }
+
+        alreadyVisited.insert((void*)p);
+
+        if (outSlabs && Slab::slabForAlloc(p)) {
+            outSlabs->insert(Slab::slabForAlloc(p));
+            return 0;
+        }
+
+        HeldClass* actualHeldClassType = p->vtable->mType;
+
+        return actualHeldClassType->deepBytecount(p->data, alreadyVisited, outSlabs) +
+            bytesRequiredForAllocation(sizeof(layout) + actualHeldClassType->bytecount());
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t inWireType, bool asIfFinal=false) {
         if (isFinal() || asIfFinal) {
@@ -211,7 +277,7 @@ public:
 
                     initializeInstance(
                         self,
-                        (layout*)malloc(
+                        (layout*)tp_malloc(
                             sizeof(layout) + m_heldClass->bytecount()
                         ),
                         0
@@ -313,7 +379,7 @@ public:
     // 'initializer(instance_ptr memberData, int memberIx)' for each member
     template<class sub_constructor>
     void constructor(instance_ptr self, const sub_constructor& initializer) const {
-        initializeInstance(self, (layout*)malloc(sizeof(layout) + m_heldClass->bytecount()), 0);
+        initializeInstance(self, (layout*)tp_malloc(sizeof(layout) + m_heldClass->bytecount()), 0);
 
         layout& l = *instanceToLayout(self);
         l.refcount = 1;
@@ -322,7 +388,7 @@ public:
         try {
             m_heldClass->constructor(l.data, initializer);
         } catch (...) {
-            free(instanceToLayout(self));
+            tp_free(instanceToLayout(self));
         }
     }
 
@@ -330,7 +396,7 @@ public:
     // 'initializer(instance_ptr data)'
     template<class sub_constructor>
     void constructorInitializingHeld(instance_ptr self, const sub_constructor& initializer) const {
-        initializeInstance(self, (layout*)malloc(sizeof(layout) + m_heldClass->bytecount()), 0);
+        initializeInstance(self, (layout*)tp_malloc(sizeof(layout) + m_heldClass->bytecount()), 0);
 
         layout& l = *instanceToLayout(self);
         l.refcount = 1;
@@ -339,7 +405,7 @@ public:
         try {
             initializer(l.data);
         } catch (...) {
-            free(instanceToLayout(self));
+            tp_free(instanceToLayout(self));
         }
     }
 
@@ -353,6 +419,10 @@ public:
 
     void assign(instance_ptr self, instance_ptr other);
 
+    bool getMemberIsNonempty(int index) const {
+        return m_heldClass->getMemberIsNonempty(index);
+    }
+
     Type* getMemberType(int index) const {
         return m_heldClass->getMemberType(index);
     }
@@ -369,7 +439,7 @@ public:
         return m_heldClass->getMemberFunctionMethodType(name, false);
     }
 
-    const std::vector<std::tuple<std::string, Type*, Instance> >& getMembers() const {
+    const std::vector<MemberDefinition>& getMembers() const {
         return m_heldClass->getMembers();
     }
 
@@ -389,7 +459,7 @@ public:
         return m_heldClass->getPropertyFunctions();
     }
 
-    const std::vector<std::tuple<std::string, Type*, Instance> >& getOwnMembers() const {
+    const std::vector<MemberDefinition>& getOwnMembers() const {
         return m_heldClass->getOwnMembers();
     }
 
