@@ -31,6 +31,17 @@ public:
         endOfConstructorInitialization(); // finish initializing the type object.
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        return m_alternative->deepcopy(dest, src, context);
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        return m_alternative->deepBytecount(instance, alreadyVisited, outSlabs);
+    }
 
     void _updateTypeMemosAfterForwardResolution() {
         ConcreteAlternative::Make(m_alternative, m_which, this);
@@ -91,7 +102,7 @@ public:
             *(uint8_t*)self = m_which;
             s(self);
         } else {
-            *(layout**)self = (layout*)malloc(
+            *(layout**)self = (layout*)tp_malloc(
                 sizeof(layout) +
                 elementType()->bytecount()
                 );
@@ -102,7 +113,7 @@ public:
             try {
                 s(record.data);
             } catch(...) {
-                free(*(layout**)self);
+                tp_free(*(layout**)self);
                 throw;
             }
         }
