@@ -37,7 +37,10 @@ class ConstDictType : public Type {
         uint8_t data[];
     };
 
+    typedef layout* layout_ptr;
+
 public:
+
     ConstDictType(Type* key, Type* value) :
             Type(TypeCategory::catConstDict),
             m_key(key),
@@ -125,6 +128,119 @@ public:
         buffer.writeEndCompound();
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        layout_ptr& destRecordPtr = *(layout**)dest;
+        layout_ptr& srcRecordPtr = *(layout**)src;
+
+        if (!srcRecordPtr) {
+            destRecordPtr = srcRecordPtr;
+            return;
+        }
+
+        auto doDeepcopy = [&]() {
+            int bytecount;
+            if (srcRecordPtr->subpointers) {
+                bytecount = sizeof(layout) + srcRecordPtr->subpointers * m_bytes_per_key_subtree_pair;
+            } else {
+                bytecount = sizeof(layout) + srcRecordPtr->count * m_bytes_per_key_value_pair;
+            }
+
+            destRecordPtr = (layout_ptr)context.slab->allocate(bytecount, this);
+
+            destRecordPtr->count = srcRecordPtr->count;
+            destRecordPtr->subpointers = srcRecordPtr->subpointers;
+            destRecordPtr->refcount = 0;
+            destRecordPtr->hash_cache = srcRecordPtr->hash_cache;
+
+            if (srcRecordPtr->subpointers) {
+                for (long k = 0; k < srcRecordPtr->subpointers; k++) {
+                    m_key->deepcopy(
+                        kdPairPtrKey(dest, k),
+                        kdPairPtrKey(src, k),
+                        context
+                    );
+                    this->deepcopy(
+                        kdPairPtrDict(dest, k),
+                        kdPairPtrDict(src, k),
+                        context
+                    );
+                }
+            } else {
+                for (long k = 0; k < srcRecordPtr->count; k++) {
+                    m_key->deepcopy(
+                        kvPairPtrKey(dest, k),
+                        kvPairPtrKey(src, k),
+                        context
+                    );
+                    m_value->deepcopy(
+                        kvPairPtrValue(dest, k),
+                        kvPairPtrValue(src, k),
+                        context
+                    );
+                }
+            }
+        };
+
+        if (srcRecordPtr->refcount == 1) {
+            doDeepcopy();
+        } else {
+            auto it = context.alreadyAllocated.find((instance_ptr)srcRecordPtr);
+            if (it == context.alreadyAllocated.end()) {
+                doDeepcopy();
+                context.alreadyAllocated[(instance_ptr)srcRecordPtr] = (instance_ptr)destRecordPtr;
+            } else {
+                destRecordPtr = (layout_ptr)it->second;
+            }
+        }
+
+        destRecordPtr->refcount++;
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        layout* l = *(layout**)instance;
+
+        if (!l) {
+            return 0;
+        }
+
+        if (l->refcount != 1) {
+            if (alreadyVisited.find((void*)l) != alreadyVisited.end()) {
+                return 0;
+            }
+        }
+
+        alreadyVisited.insert((void*)l);
+
+        if (outSlabs && Slab::slabForAlloc(l)) {
+            outSlabs->insert(Slab::slabForAlloc(l));
+            return 0;
+        }
+
+        size_t res = bytesRequiredForAllocation(
+            sizeof(layout)
+            + l->subpointers * m_bytes_per_key_subtree_pair
+            + l->count * m_bytes_per_key_value_pair
+            );
+
+        if (l->subpointers) {
+            for (long k = 0; k < l->subpointers; k++) {
+                res += m_key->deepBytecount(kdPairPtrKey(instance, k), alreadyVisited, outSlabs);
+                res += deepBytecount(kdPairPtrDict(instance, k), alreadyVisited, outSlabs);
+            }
+        } else {
+            for (long k = 0; k < l->count; k++) {
+                res += m_key->deepBytecount(kvPairPtrKey(instance, k), alreadyVisited, outSlabs);
+                res += m_value->deepBytecount(kvPairPtrValue(instance, k), alreadyVisited, outSlabs);
+            }
+        }
+
+        return res;
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         int32_t ct = -1;
@@ -175,6 +291,10 @@ public:
 
     void subtractTupleOfKeysFromDict(instance_ptr lhs, instance_ptr rhs, instance_ptr output);
 
+    instance_ptr kdPairPtrKey(instance_ptr self, int64_t i);
+
+    instance_ptr kdPairPtrDict(instance_ptr self, int64_t i);
+
     instance_ptr kvPairPtrKey(instance_ptr self, int64_t i);
 
     instance_ptr kvPairPtrValue(instance_ptr self, int64_t i);
