@@ -40,7 +40,6 @@ public:
         endOfConstructorInitialization(); // finish initializing the type object.
     }
 
-
     void _updateTypeMemosAfterForwardResolution() {
         DictType::Make(m_key, m_value, this);
     }
@@ -113,6 +112,72 @@ public:
         }
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        hash_table_layout_ptr& destRecordPtr = *(hash_table_layout**)dest;
+        hash_table_layout_ptr& srcRecordPtr = *(hash_table_layout**)src;
+
+        auto it = context.alreadyAllocated.find((instance_ptr)srcRecordPtr);
+
+        if (it == context.alreadyAllocated.end()) {
+            destRecordPtr = srcRecordPtr->deepcopy(
+            context,
+                this,
+                m_key,
+                m_value
+            );
+
+            context.alreadyAllocated[(instance_ptr)srcRecordPtr] = (instance_ptr)destRecordPtr;
+        } else {
+            destRecordPtr = (hash_table_layout_ptr)context.alreadyAllocated[(instance_ptr)srcRecordPtr];
+            destRecordPtr->refcount++;
+        }
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        hash_table_layout& l = **(hash_table_layout**)instance;
+
+        if (alreadyVisited.find((void*)&l) != alreadyVisited.end()) {
+            return 0;
+        }
+
+        alreadyVisited.insert((void*)&l);
+
+        if (outSlabs && Slab::slabForAlloc(&l)) {
+            outSlabs->insert(Slab::slabForAlloc(&l));
+            return 0;
+        }
+
+        size_t res = bytesRequiredForAllocation(sizeof(hash_table_layout));
+        // count 'items_populated' and 'items'
+        res += bytesRequiredForAllocation(l.items_reserved);
+        res += bytesRequiredForAllocation(l.items_reserved * m_bytes_per_key_value_pair);
+
+        // count the hashtable
+        res += bytesRequiredForAllocation(sizeof(int32_t) * l.hash_table_size) * 2;
+
+        if (!m_key->isPOD()) {
+            for (long k = 0; k < l.items_reserved; k++) {
+                if (l.items_populated[k]) {
+                    res += m_key->deepBytecount(l.items + k * m_bytes_per_key_value_pair, alreadyVisited, outSlabs);
+                }
+            }
+        }
+
+        if (!m_value->isPOD()) {
+            for (long k = 0; k < l.items_reserved; k++) {
+                if (l.items_populated[k]) {
+                    res += m_value->deepBytecount(l.items + k * m_bytes_per_key_value_pair + m_bytes_per_key, alreadyVisited, outSlabs);
+                }
+            }
+        }
+
+        return res;
+    }
+
     template<class buf_t>
     void serialize(instance_ptr self, buf_t& buffer, size_t fieldNumber) {
         hash_table_layout& l = **(hash_table_layout**)self;
@@ -149,8 +214,8 @@ public:
 
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
-        size_t count;
-        size_t id;
+        size_t count = 0;
+        size_t id = 0;
         bool wasFromId = false;
 
         size_t valuesRead = buffer.consumeCompoundMessageWithImpliedFieldNumbers(wireType,
