@@ -1030,6 +1030,52 @@ public:
             });
         }
 
+        static void extractGlobalAccessesFromCode(PyCodeObject* code, std::set<std::string>& outAccesses) {
+            uint8_t* bytes;
+            Py_ssize_t bytecount;
+
+            PyBytes_AsStringAndSize(((PyCodeObject*)code)->co_code, (char**)&bytes, &bytecount);
+
+            long opcodeCount = bytecount / 2;
+
+            // opcodes are encoded in the low byte
+            auto opcodeFor = [&](int i) { return bytes[i * 2]; };
+
+            // opcode targets are encoded in the high byte
+            auto opcodeTargetFor = [&](int i) { return bytes[i * 2 + 1]; };
+
+            const uint8_t LOAD_GLOBAL = 116;
+            const uint8_t DELETE_GLOBAL = 98;
+            const uint8_t STORE_GLOBAL = 97;
+
+            for (long ix = 0; ix < opcodeCount; ix++) {
+                // if we're loading a global, we start a new sequence
+                if (opcodeFor(ix) == LOAD_GLOBAL) {
+                    PyObject* name = PyTuple_GetItem(code->co_names, opcodeTargetFor(ix));
+                    if (!PyUnicode_Check(name)) {
+                        throw std::runtime_error("Function had a non-string object in co_names");
+                    }
+                    outAccesses.insert(PyUnicode_AsUTF8(name));
+                } else if (
+                    opcodeFor(ix) == STORE_GLOBAL
+                    || opcodeFor(ix) == DELETE_GLOBAL
+                ) {
+                    PyObject* name = PyTuple_GetItem(code->co_names, opcodeTargetFor(ix));
+                    if (!PyUnicode_Check(name)) {
+                        throw std::runtime_error("Function had a non-string object in co_names");
+                    }
+                    outAccesses.insert(PyUnicode_AsUTF8(name));
+                }
+            }
+
+            // recurse into sub code objects
+            iterate(code->co_consts, [&](PyObject* o) {
+                if (PyCode_Check(o)) {
+                    extractGlobalAccessesFromCode((PyCodeObject*)o, outAccesses);
+                }
+            });
+        }
+
         static void extractNamesFromCode(PyCodeObject* code, std::set<PyObject*>& outNames) {
             iterate(code->co_names, [&](PyObject* o) { outNames.insert(o); });
             iterate(code->co_freevars, [&](PyObject* o) { outNames.insert(o); });
@@ -1073,6 +1119,9 @@ public:
             PyObject *key, *value;
             Py_ssize_t pos = 0;
 
+            // place them in sorted order
+            std::map<std::string, std::pair<PyObject*, PyObject*> > toCopy;
+
             while (PyDict_Next(mFunctionGlobals, &pos, &key, &value)) {
                 if (PyUnicode_Check(key)) {
                     std::string globalName = PyUnicode_AsUTF8(key);
@@ -1084,13 +1133,17 @@ public:
                     }
 
                     if (allNamesString.find(shortGlobalName) != allNamesString.end()) {
-                        if (PyDict_SetItem(result, key, value)) {
-                            throw PythonExceptionSet();
-                        }
+                        toCopy[globalName] = std::make_pair(key, value);
                     }
                 }
             }
 
+            for (auto& nameandKV: toCopy) {
+                if (PyDict_SetItem(result, nameandKV.second.first, nameandKV.second.second)) {
+                    throw PythonExceptionSet();
+                }
+            }
+
             PyObject* builtins = PyDict_GetItemString(mFunctionGlobals, "__builtins__");
             if (builtins) {
                 PyDict_SetItemString(result, "__builtins__", builtins);
@@ -1501,6 +1554,22 @@ public:
         return mClosureType->cmp(left, right, pyComparisonOp, suppressExceptions);
     }
 
+
+
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        // we don't deepcopy into functions
+        copy_constructor(dest, src);
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        // we explicitly don't count functions
+        return 0;
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         mClosureType->deserialize(self, buffer, wireType);
