@@ -61,6 +61,49 @@ public:
     }
 };
 
+// models a single class member definition
+class MemberDefinition {
+public:
+    MemberDefinition(
+        const std::string& inName,
+        Type* inType,
+        const Instance& inDefaultValue,
+        bool nonempty
+    ) :
+        mName(inName),
+        mType(inType),
+        mDefaultValue(inDefaultValue),
+        mIsNonempty(nonempty)
+    {}
+
+    const std::string& getName() const {
+        return mName;
+    }
+
+    Type*& getType() {
+        return mType;
+    }
+
+    Type* getType() const {
+        return mType;
+    }
+
+    const Instance& getDefaultValue() const {
+        return mDefaultValue;
+    }
+
+    bool getIsNonempty() const {
+        return mIsNonempty;
+    }
+
+private:
+    std::string mName;
+    Type* mType;
+    Instance mDefaultValue;
+    bool mIsNonempty;
+};
+
+
 /****
 ClassDispatchTable
 
@@ -102,7 +145,7 @@ public:
         mFuncPtrsUsed = baseAsBase->mFuncPtrsUsed;
         mFuncPtrsAllocated = baseAsBase->mFuncPtrsAllocated;
 
-        mFuncPtrs = (untyped_function_ptr*)malloc(sizeof(untyped_function_ptr) * mFuncPtrsAllocated);
+        mFuncPtrs = (untyped_function_ptr*)tp_malloc(sizeof(untyped_function_ptr) * mFuncPtrsAllocated);
 
         for (long k = 0; k < mFuncPtrsUsed; k++) {
             mFuncPtrs[k] = nullptr;
@@ -132,7 +175,7 @@ public:
         // compiled code doesn't have to do that.
         if (mFuncPtrsUsed >= mFuncPtrsAllocated) {
             mFuncPtrsAllocated = (mFuncPtrsAllocated + 1) * 2;
-            untyped_function_ptr* newTable = (untyped_function_ptr*)malloc(sizeof(untyped_function_ptr) * mFuncPtrsAllocated);
+            untyped_function_ptr* newTable = (untyped_function_ptr*)tp_malloc(sizeof(untyped_function_ptr) * mFuncPtrsAllocated);
             for (long k = 0; k < mFuncPtrsUsed; k++) {
                 newTable[k] = mFuncPtrs[k];
             }
@@ -281,6 +324,8 @@ public:
     // members are in this particular layout, so that we know how far ahead to look in
     // the object when we are looking up a particular data member.
     int64_t mInitializationBitByteCount;
+
+    Type* mClassType;
 };
 
 //a class held directly inside of another object
@@ -289,7 +334,7 @@ public:
     HeldClass(std::string inName,
           const std::vector<HeldClass*>& baseClasses,
           bool isFinal,
-          const std::vector<std::tuple<std::string, Type*, Instance> >& members,
+          const std::vector<MemberDefinition>& members,
           const std::map<std::string, Function*>& memberFunctions,
           const std::map<std::string, Function*>& staticFunctions,
           const std::map<std::string, Function*>& propertyFunctions,
@@ -307,6 +352,9 @@ public:
             m_own_classMembers(classMembers),
             m_hasComparisonOperators(false),
             m_hasGetAttributeMagicMethod(false),
+            m_hasGetAttrMagicMethod(false),
+            m_hasSetAttrMagicMethod(false),
+            m_hasDelAttrMagicMethod(false),
             m_refToType(nullptr)
     {
         m_name = inName;
@@ -322,9 +370,10 @@ public:
 
         res += ShaHash(1);
         for (auto tup: m_own_members) {
-            res += ShaHash(std::get<0>(tup));
-            res += ShaHash(std::get<1>(tup)->identityHash(groupHead));
-            res += MutuallyRecursiveTypeGroup::tpInstanceShaHash(std::get<2>(tup), groupHead);
+            res += ShaHash(tup.getName());
+            res += ShaHash(tup.getType()->identityHash(groupHead));
+            res += MutuallyRecursiveTypeGroup::tpInstanceShaHash(tup.getDefaultValue(), groupHead);
+            res += ShaHash(tup.getIsNonempty());
         }
 
         res += ShaHash(2);
@@ -357,7 +406,7 @@ public:
     template<class visitor_type>
     void _visitCompilerVisibleInstances(const visitor_type& visitor) {
         for (auto tup: m_own_members) {
-            visitor(std::get<2>(tup));
+            visitor(tup.getDefaultValue());
         }
     }
 
@@ -366,7 +415,7 @@ public:
     template<class visitor_type>
     void _visitContainedTypes(const visitor_type& visitor) {
         for (auto& o: m_members) {
-            visitor(std::get<1>(o));
+            visitor(o.getType());
         }
     }
 
@@ -380,40 +429,44 @@ public:
             }
         }
         for (auto& o: m_own_members) {
-            visitor(std::get<1>(o));
+            // this is expected to actually modify the type
+            // if its a forward.
+            visitor(o.getType());
         }
         for (auto& o: m_own_memberFunctions) {
-            Type* t = std::get<1>(o);
+            Type* t = o.second;
             visitor(t);
-            assert(t == std::get<1>(o));
+            assert(t == o.second);
         }
         for (auto& o: m_own_staticFunctions) {
-            Type* t = std::get<1>(o);
+            Type* t = o.second;
             visitor(t);
-            assert(t == std::get<1>(o));
+            assert(t == o.second);
         }
         for (auto& o: m_own_propertyFunctions) {
-            Type* t = std::get<1>(o);
+            Type* t = o.second;
             visitor(t);
-            assert(t == std::get<1>(o));
+            assert(t == o.second);
         }
         for (auto& o: m_members) {
-            visitor(std::get<1>(o));
+            // this is expected to actually modify the type
+            // if its a forward.
+            visitor(o.getType());
         }
         for (auto& o: m_memberFunctions) {
-            Type* t = std::get<1>(o);
+            Type* t = o.second;
             visitor(t);
-            assert(t == std::get<1>(o));
+            assert(t == o.second);
         }
         for (auto& o: m_staticFunctions) {
-            Type* t = std::get<1>(o);
+            Type* t = o.second;
             visitor(t);
-            assert(t == std::get<1>(o));
+            assert(t == o.second);
         }
         for (auto& o: m_propertyFunctions) {
-            Type* t = std::get<1>(o);
+            Type* t = o.second;
             visitor(t);
-            assert(t == std::get<1>(o));
+            assert(t == o.second);
         }
     }
 
@@ -430,7 +483,7 @@ public:
         std::string inName,
         const std::vector<HeldClass*>& bases,
         bool isFinal,
-        const std::vector<std::tuple<std::string, Type*, Instance> >& members,
+        const std::vector<MemberDefinition>& members,
         const std::map<std::string, Function*>& memberFunctions,
         const std::map<std::string, Function*>& staticFunctions,
         const std::map<std::string, Function*>& propertyFunctions,
@@ -443,6 +496,7 @@ public:
             throw std::runtime_error("Class is already set.");
         }
         m_classType = inClass;
+        m_vtable->mClassType = (Type*)inClass;
     }
 
     Class* getClassType() const {
@@ -472,6 +526,33 @@ public:
 
     bool cmp(instance_ptr left, instance_ptr right, int pyComparisonOp, bool suppressExceptions);
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        for (int64_t k = 0; k < m_members.size(); k++) {
+            if (checkInitializationFlag(src, k)) {
+                m_members[k].getType()->deepcopy(
+                    eltPtr(dest, k),
+                    eltPtr(src, k),
+                    context
+                );
+                setInitializationFlag(dest, k);
+            }
+        }
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        size_t res = 0;
+
+        for (long k = 0; k < m_members.size(); k++) {
+            res += m_members[k].getType()->deepBytecount(eltPtr(instance, k), alreadyVisited, outSlabs);
+        }
+
+        return res;
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         for (long k = 0; k < m_members.size();k++) {
@@ -495,7 +576,7 @@ public:
         for (long k = 0; k < m_members.size();k++) {
             bool isInitialized = checkInitializationFlag(self, k);
             if (isInitialized) {
-                std::get<1>(m_members[k])->serialize(eltPtr(self,k),buffer, k);
+                m_members[k].getType()->serialize(eltPtr(self,k),buffer, k);
             }
         }
 
@@ -514,13 +595,15 @@ public:
                 setInitializationFlag(self, k);
             } catch(...) {
                 for (long k2 = k-1; k2 >= 0; k2--) {
-                    std::get<1>(m_members[k2])->destroy(eltPtr(self,k2));
+                    m_members[k2].getType()->destroy(eltPtr(self,k2));
                 }
                 throw;
             }
         }
     }
 
+    void delAttribute(instance_ptr self, int memberIndex) const;
+
     void setAttribute(instance_ptr self, int memberIndex, instance_ptr other) const;
 
     //don't default construct classes
@@ -528,7 +611,7 @@ public:
         return t->is_default_constructible() && t->getTypeCategory() != TypeCategory::catClass;
     }
 
-    void constructor(instance_ptr self);
+    void constructor(instance_ptr self, bool allowEmpty=false);
 
     void destroy(instance_ptr self);
 
@@ -536,12 +619,12 @@ public:
 
     void assign(instance_ptr self, instance_ptr other);
 
-    bool checkInitializationFlag(instance_ptr self, int memberIndex) const {
-        int byte = memberIndex / 8;
-        int bit = memberIndex % 8;
-        return bool( ((uint8_t*)self)[byte] & (1 << bit) );
+    bool fieldGuaranteedInitialized(int ix) const {
+        return m_members[ix].getIsNonempty();
     }
 
+    bool checkInitializationFlag(instance_ptr self, int memberIndex) const;
+
     bool isFinal() {
         return m_is_final;
     }
@@ -550,31 +633,35 @@ public:
 
     void clearInitializationFlag(instance_ptr self, int memberIndex) const;
 
+    bool getMemberIsNonempty(int index) const {
+        return m_members[index].getIsNonempty();
+    }
+
     Type* getMemberType(int index) const {
-        return std::get<1>(m_members[index]);
+        return m_members[index].getType();
     }
 
     const std::string& getMemberName(int index) const {
-        return std::get<0>(m_members[index]);
+        return m_members[index].getName();
     }
 
     bool memberHasDefaultValue(int index) const {
-        return std::get<2>(m_members[index]).type()->getTypeCategory() != TypeCategory::catNone;
+        return m_members[index].getDefaultValue().type()->getTypeCategory() != TypeCategory::catNone;
     }
 
     const Instance& getMemberDefaultValue(int index) const {
-        return std::get<2>(m_members[index]);
+        return m_members[index].getDefaultValue();
     }
 
     const std::vector<HeldClass*> getBases() const {
         return m_bases;
     }
 
-    const std::vector<std::tuple<std::string, Type*, Instance> >& getMembers() const {
+    const std::vector<MemberDefinition>& getMembers() const {
         return m_members;
     }
 
-    const std::vector<std::tuple<std::string, Type*, Instance> >& getOwnMembers() const {
+    const std::vector<MemberDefinition>& getOwnMembers() const {
         return m_own_members;
     }
 
@@ -634,6 +721,18 @@ public:
         return m_hasGetAttributeMagicMethod;
     }
 
+    bool hasGetAttrMagicMethod() const {
+        return m_hasGetAttrMagicMethod;
+    }
+
+    bool hasSetAttrMagicMethod() const {
+        return m_hasSetAttrMagicMethod;
+    }
+
+    bool hasDelAttrMagicMethod() const {
+        return m_hasDelAttrMagicMethod;
+    }
+
     void initializeMRO() {
         // this is not how the MRO actually works, but we have yet to actually
         // code it correctly.
@@ -682,15 +781,15 @@ public:
         }
 
         for (auto nameAndType: m_members) {
-            membersSoFar.insert(std::get<0>(nameAndType));
+            membersSoFar.insert(nameAndType.getName());
         }
 
         for (auto nameAndType: m_own_members) {
-            if (membersSoFar.find(std::get<0>(nameAndType)) != membersSoFar.end()) {
-                throw std::runtime_error("Can't redefine member named " + std::get<0>(nameAndType));
+            if (membersSoFar.find(nameAndType.getName()) != membersSoFar.end()) {
+                throw std::runtime_error("Can't redefine member named " + nameAndType.getName());
             }
 
-            membersSoFar.insert(std::get<0>(nameAndType));
+            membersSoFar.insert(nameAndType.getName());
 
             m_members.push_back(nameAndType);
         }
@@ -699,7 +798,7 @@ public:
             // note that we explicitly leak the string so that the refcount on c_str
             // stays active. I'm sure there's a better way to do this, but types are
             // permanent, so we would never have cleaned this up anyways.
-            m_membersByName[(new std::string(std::get<0>(m_members[k])))->c_str()] = k;
+            m_membersByName[(new std::string(m_members[k].getName()))->c_str()] = k;
         }
 
         for (HeldClass* ancestor: m_mro) {
@@ -710,7 +809,13 @@ public:
             ancestor->m_implementors.insert(this);
         }
 
-        m_vtable->finalize(&mClassDispatchTables[0], (m_members.size() + 7) / 8, mClassDispatchTables.size());
+        updateBytesOfInitBits();
+
+        m_vtable->finalize(
+            &mClassDispatchTables[0],
+            mBytesOfInitializationBits,
+            mClassDispatchTables.size()
+        );
 
         // make sure that, for every interface we can take on, we have slots allocated
         // that the compiler can come along and compile.
@@ -721,15 +826,39 @@ public:
         setMagicMethodExistConstants();
     }
 
-    void setMagicMethodExistConstants() {
-        if (m_memberFunctions.find("__eq__") != m_memberFunctions.end()) { m_hasComparisonOperators = true; }
-        if (m_memberFunctions.find("__ne__") != m_memberFunctions.end()) { m_hasComparisonOperators = true; }
-        if (m_memberFunctions.find("__lt__") != m_memberFunctions.end()) { m_hasComparisonOperators = true; }
-        if (m_memberFunctions.find("__gt__") != m_memberFunctions.end()) { m_hasComparisonOperators = true; }
-        if (m_memberFunctions.find("__le__") != m_memberFunctions.end()) { m_hasComparisonOperators = true; }
-        if (m_memberFunctions.find("__ge__") != m_memberFunctions.end()) { m_hasComparisonOperators = true; }
+    void updateBytesOfInitBits();
 
-        if (m_memberFunctions.find("__getattribute__") != m_memberFunctions.end()) { m_hasGetAttributeMagicMethod = true; }
+    void setMagicMethodExistConstants() {
+        if (m_memberFunctions.find("__eq__") != m_memberFunctions.end()) {
+            m_hasComparisonOperators = true;
+        }
+        if (m_memberFunctions.find("__ne__") != m_memberFunctions.end()) {
+            m_hasComparisonOperators = true;
+        }
+        if (m_memberFunctions.find("__lt__") != m_memberFunctions.end()) {
+            m_hasComparisonOperators = true;
+        }
+        if (m_memberFunctions.find("__gt__") != m_memberFunctions.end()) {
+            m_hasComparisonOperators = true;
+        }
+        if (m_memberFunctions.find("__le__") != m_memberFunctions.end()) {
+            m_hasComparisonOperators = true;
+        }
+        if (m_memberFunctions.find("__ge__") != m_memberFunctions.end()) {
+            m_hasComparisonOperators = true;
+        }
+        if (m_memberFunctions.find("__getattribute__") != m_memberFunctions.end()) {
+            m_hasGetAttributeMagicMethod = true;
+        }
+        if (m_memberFunctions.find("__delattr__") != m_memberFunctions.end()) {
+            m_hasDelAttrMagicMethod = true;
+        }
+        if (m_memberFunctions.find("__setattr__") != m_memberFunctions.end()) {
+            m_hasSetAttrMagicMethod = true;
+        }
+        if (m_memberFunctions.find("__getattr__") != m_memberFunctions.end()) {
+            m_hasGetAttrMagicMethod = true;
+        }
     }
 
     size_t allocateMethodDispatch(std::string funcName, function_call_signature_type signature) {
@@ -811,6 +940,13 @@ public:
             return it->second;
         }
 
+        // check if an 'equivalent' class is floating around in there.
+        for (long k = 0; k < m_mro.size(); k++) {
+            if (Type::typesEquivalent(m_mro[k], ancestor)) {
+                return k;
+            }
+        }
+
         return -1;
     }
 
@@ -833,6 +969,8 @@ private:
 
     std::vector<HeldClass*> m_bases;
 
+    size_t mBytesOfInitializationBits;
+
     // if final, we can't subclass this class
     bool m_is_final;
 
@@ -853,7 +991,7 @@ private:
     std::vector<ClassDispatchTable> mClassDispatchTables;
 
     //the members we
-    std::vector<std::tuple<std::string, Type*, Instance> > m_members;
+    std::vector<MemberDefinition> m_members;
 
     std::map<std::string, Function*> m_memberFunctions;
 
@@ -864,7 +1002,7 @@ private:
     std::map<std::string, PyObject*> m_classMembers;
 
     // the original members we were provided with
-    std::vector<std::tuple<std::string, Type*, Instance> > m_own_members;
+    std::vector<MemberDefinition> m_own_members;
 
     std::map<std::string, Function*> m_own_memberFunctions;
 
@@ -880,4 +1018,7 @@ private:
 
     bool m_hasComparisonOperators;
     bool m_hasGetAttributeMagicMethod;
+    bool m_hasGetAttrMagicMethod;
+    bool m_hasSetAttrMagicMethod;
+    bool m_hasDelAttrMagicMethod;
 };
