@@ -77,6 +77,25 @@ public:
 
     std::string computeName() const;
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        uint8_t which = *(uint8_t*)dest = *(uint8_t*)src;
+        m_types[which]->deepcopy(dest+1, src+1, context);
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        if (isPOD()) {
+            return 0;
+        }
+
+        int fieldNumber = *(uint8_t*)instance;
+
+        return m_types[fieldNumber]->deepBytecount(instance + 1, alreadyVisited, outSlabs);
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         bool hitOne = false;
