@@ -1,17 +1,17 @@
 /******************************************************************************
-   Copyright 2017-2019 typed_python Authors
+    Copyright 2017-2020 typed_python Authors
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-       http://www.apache.org/licenses/LICENSE-2.0
+      http://www.apache.org/licenses/LICENSE-2.0
 
-   Unless required by applicable law or agreed to in writing, software
-   distributed under the License is distributed on an "AS IS" BASIS,
-   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-   See the License for the specific language governing permissions and
-   limitations under the License.
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 ******************************************************************************/
 
 #pragma once
@@ -27,8 +27,9 @@ public:
 
     ~PyEnsureGilReleased();
 
+    static void gilReleaseThreadLoop();
 private:
-    bool m_should_reaquire;
+    bool m_should_reacquire;
 };
 
 //scoped object to ensure we're holding the GIL. If
@@ -44,4 +45,5 @@ private:
     bool m_should_rerelease;
 };
 
+
 void assertHoldingTheGil();
