@@ -68,13 +68,27 @@ class PyInstance {
 public:
     PyObject_HEAD
 
-    bool mIsInitialized;
     char mIteratorFlag; //0 is keys, 1 is values, 2 is pairs
     int64_t mIteratorOffset; //-1 if we're not an iterator
     int64_t mContainerSize; //-1 if we're not an iterator
 
     Instance mContainingInstance;
 
+    // we may be a temporary ref to a type
+    instance_ptr mTemporaryRefTo;
+
+    // initialize our fields after we have called 'tp_alloc'
+    void initializeEmpty() {
+        mIteratorFlag = -1;
+        mIteratorOffset = -1;
+        mContainerSize = -1;
+        mTemporaryRefTo = nullptr;
+
+        new (&mContainingInstance) Instance();
+    }
+
+    void resolveTemporaryReference();
+
     template<class T>
     static auto specialize(PyObject* obj, const T& f, Type* typeOverride = nullptr) {
         if (!typeOverride) {
@@ -298,7 +312,7 @@ public:
         PyInstance* self =
             (PyInstance*)typeObj(eltType)->tp_alloc(typeObj(eltType), 0);
 
-        self->mIteratorOffset = -1;
+        self->initializeEmpty();
 
         try {
             self->initialize(f, eltType);
@@ -310,14 +324,24 @@ public:
         }
     }
 
+    static PyObject* initializeTemporaryRef(Type* eltType, instance_ptr data) {
+        eltType->assertForwardsResolvedSufficientlyToInstantiate();
+
+        PyInstance* self =
+            (PyInstance*)typeObj(eltType)->tp_alloc(typeObj(eltType), 0);
+
+        self->initializeEmpty();
+        self->mTemporaryRefTo = data;
+
+        return (PyObject*)self;
+    }
+
     template<class init_func>
     void initialize(const init_func& i, Type* typeIfKnown = nullptr) {
         Type* type = typeIfKnown ? typeIfKnown : extractTypeFrom(((PyObject*)this)->ob_type);
         type->assertForwardsResolvedSufficientlyToInstantiate();
 
-        mIsInitialized = false;
-        new (&mContainingInstance) Instance( type, i );
-        mIsInitialized = true;
+        mContainingInstance = Instance(type, i);
     }
 
     static PyObject* fromInstance(const Instance& instance) {
@@ -357,9 +381,9 @@ public:
     static bool pyValCouldBeOfType(Type* t, PyObject* pyRepresentation, ConversionLevel level);
 
     /**
-     construct an 'eltType' from a python object at 'tgt'. If 'isExplicit' then we're invoked from an explicit
-     copy constructor, so more liberal conversion is allowed than if 'isExplicit' is false, which happens
-     when we're attempting to convert for purposes of method dispatch.
+     construct an 'eltType' from a python object at 'tgt'. ConversionLevel determines what
+     level of type conversion we're willing to tolerate. If we can't do the conversion, we'll
+     throw an exception.
      */
     static void copyConstructFromPythonInstance(Type* eltType, instance_ptr tgt, PyObject* pyRepresentation, ConversionLevel level);
 
@@ -372,7 +396,9 @@ public:
     //produce the pythonic representation of this object. for values that have a direct python representation,
     //such as integers, strings, bools, or None, we return an actual python object. Otherwise,
     //we return a pointer to a PyInstance representing the object.
-    static PyObject* extractPythonObject(instance_ptr data, Type* eltType);
+    //if 'createTemporaryRef' is 'false', then we don't create temporary references to HeldClass
+    //instances.
+    static PyObject* extractPythonObject(instance_ptr data, Type* eltType, bool createTemporaryRef=true);
 
     static PyObject* extractPythonObject(const Instance& instance);
 
