@@ -31,6 +31,8 @@ public:
         PyObject* pyObj;
     };
 
+    typedef layout_type* layout_ptr;
+
     // return a new layout with a refcount of 1, but steal the reference
     // to the python object.
     static layout_type* stealToCreateLayout(PyObject* p) {
@@ -40,7 +42,7 @@ public:
     // return a new layout with a refcount of 1, increffing the argument
     // before placing it in the layout.
     static layout_type* createLayout(PyObject* p, bool alsoIncref = true) {
-        layout_type* res = (layout_type*)malloc(sizeof(layout_type));
+        layout_type* res = (layout_type*)tp_malloc(sizeof(layout_type));
 
         if (alsoIncref) {
             incref(p);
@@ -52,8 +54,8 @@ public:
         return res;
     }
 
-    void initializeHandleAt(instance_ptr ptr, int refcount=1) {
-        ((layout_type**)ptr)[0] = (layout_type*)malloc(sizeof(layout_type));
+    void initializeHandleAt(instance_ptr ptr) {
+        ((layout_type**)ptr)[0] = (layout_type*)tp_malloc(sizeof(layout_type));
         ((layout_type**)ptr)[0]->pyObj = NULL;
         ((layout_type**)ptr)[0]->refcount = 1;
     }
@@ -75,7 +77,7 @@ public:
         if (p->refcount == 0) {
             PyEnsureGilAcquired getTheGil;
             decref(p->pyObj);
-            free(p);
+            tp_free(p);
         }
     }
 
@@ -85,7 +87,7 @@ public:
         if (getHandlePtr(self)->refcount == 0) {
             PyEnsureGilAcquired getTheGil;
             decref(getPyObj(self));
-            free(*(layout_type**)self);
+            tp_free(*(layout_type**)self);
         }
     }
 
@@ -170,12 +172,31 @@ public:
         return false;
     }
 
+    int64_t refcount(instance_ptr self) const {
+        return getHandlePtr(self)->refcount;
+    }
+
     template<class buf_t>
     void serialize(instance_ptr self, buf_t& buffer, size_t fieldNumber) {
         PyObject* p = getPyObj(self);
         buffer.getContext().serializePythonObject(p, buffer, fieldNumber);
     }
 
+    static size_t deepBytecountForPyObj(PyObject* o, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs);
+
+    static PyObject* deepcopyPyObject(
+        PyObject* o,
+        DeepcopyContext& context
+    );
+
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    );
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs);
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
          initializeHandleAt(self);
