@@ -52,35 +52,6 @@ inline void PyTuple_Set_Item_No_Checks(PyObject* o, long k, PyObject* item) {
     PyTuple_SET_ITEM(o, k, item);
 }
 
-//models bytes held in a python 'bytes' object.
-class PyBytesByteBuffer : public ByteBuffer {
-public:
-    explicit PyBytesByteBuffer(PyObject* obj) : m_obj(incref(obj)) {
-        if (!PyObject_CheckBuffer(obj)) {
-            PyErr_Format(PyExc_TypeError, "Not a buffer object.");
-            throw PythonExceptionSet();
-        }
-
-        if (PyObject_GetBuffer(obj, &m_buffer, PyBUF_SIMPLE | PyBUF_ANY_CONTIGUOUS) == -1) {
-            throw PythonExceptionSet();
-        }
-    }
-
-    virtual ~PyBytesByteBuffer() {
-        PyBuffer_Release(&m_buffer);
-        decref(m_obj);
-    }
-
-    virtual std::pair<uint8_t*, uint8_t*> range() {
-        assertHoldingTheGil();
-        return std::make_pair((uint8_t*)m_buffer.buf, (uint8_t*)m_buffer.buf + m_buffer.len);
-    }
-
-private:
-    PyObject* m_obj;
-    Py_buffer m_buffer;
-};
-
 class PythonSerializationContext : public SerializationContext {
 public:
     //enums in our protocol (serialized as uint8_t)
@@ -117,7 +88,8 @@ public:
 
     PythonSerializationContext(PyObject* typeSetObj) :
             mContextObj(typeSetObj),
-            mCompressionEnabled(false)
+            mCompressionEnabled(false),
+            mSerializeHashSequence(false)
     {
         setFlags();
     }
@@ -128,11 +100,11 @@ public:
         return mCompressionEnabled;
     }
 
-    std::shared_ptr<ByteBuffer> compress(uint8_t* begin, uint8_t* end) const;
-
-    std::shared_ptr<ByteBuffer> decompress(uint8_t* begin, uint8_t* end) const;
-
-    std::shared_ptr<ByteBuffer> compressOrDecompress(uint8_t* begin, uint8_t* end, bool compress) const;
+    // should we serialize an integer in the order of the
+    // hash sequence rather than the hash itself?
+    bool shouldSerializeHashSequence() const {
+        return mSerializeHashSequence;
+    }
 
     virtual void serializePythonObject(PyObject* o, SerializationBuffer& b, size_t fieldNumber) const;
 
@@ -144,7 +116,7 @@ public:
 
     void serializeNativeTypeInner(Type* nativeType, SerializationBuffer& b, size_t fieldNumber) const;
 
-    void serializeClassMembers(const std::vector<std::tuple<std::string, Type*, Instance> >& members, SerializationBuffer& b, int fieldNumber) const;
+    void serializeClassMembers(const std::vector<MemberDefinition>& members, SerializationBuffer& b, int fieldNumber) const;
 
     void serializeAlternativeMembers(const std::vector<std::pair<std::string, NamedTuple*> >& members, SerializationBuffer& b, int fieldNumber) const;
 
@@ -154,7 +126,7 @@ public:
 
     void deserializeAlternativeMembers(std::vector<std::pair<std::string, NamedTuple*> >& members, DeserializationBuffer& b, int wireType) const;
 
-    void deserializeClassMembers(std::vector<std::tuple<std::string, Type*, Instance> >& members, DeserializationBuffer& b, int wireType) const;
+    void deserializeClassMembers(std::vector<MemberDefinition>& members, DeserializationBuffer& b, int wireType) const;
 
     void deserializeClassFunDict(std::string className, std::map<std::string, Function*>& dict, DeserializationBuffer& b, int wireType) const;
 
@@ -220,4 +192,6 @@ private:
     bool mCompressionEnabled;
 
     bool mInternalizeTypeGroups;
+
+    bool mSerializeHashSequence;
 };
