@@ -25,17 +25,33 @@ void PythonSerializationContext::serializePythonObject(PyObject* o, Serializatio
 
     b.writeBeginCompound(fieldNumber);
 
-    // if this is already part of a recursive type group, then write it and exit
+    // if this is already part of the recursive type group that we're currently writing, and its unnamed,
+    // then write a reference to this mutually recursive type group.
+    // however, if its named, we want to fall back to the normal mechanism for writing it.
+    // we can't actually _start_ serializing a mutually recursive type group here, however,
+    // because otherwise we'd create a dependency during serialization on _whether_ the
+    // group was already created, which would introduce a nondeterminism into serilization
+    // based on the compiler state
     if (PyCell_Check(o) || PyFunction_Check(o)) {
         auto groupAndIndex = MutuallyRecursiveTypeGroup::pyObjectGroupHeadAndIndex(o, false);
 
-        if (groupAndIndex.first) {
-            b.writeBeginCompound(FieldNumbers::RECURSIVE_OBJECT);
-            serializeMutuallyRecursiveTypeGroup(groupAndIndex.first, b, 0);
-            b.writeUnsignedVarintObject(1, groupAndIndex.second);
-            b.writeEndCompound();
-            b.writeEndCompound();
-            return;
+        // if we have a group, check whether we've already memoized this group in the stream
+        // this prevents us from initiating the serialization of a new group here.
+        if (groupAndIndex.first && b.isAlreadyCached(groupAndIndex.first)) {
+            //see if the object has a name. If so, we always want to use that.
+            PyObjectStealer typeName(PyObject_CallMethod(mContextObj, "nameForObject", "(O)", o));
+            if (!typeName) {
+                throw PythonExceptionSet();
+            }
+
+            if (typeName == Py_None) {
+                b.writeBeginCompound(FieldNumbers::RECURSIVE_OBJECT);
+                serializeMutuallyRecursiveTypeGroup(groupAndIndex.first, b, 0);
+                b.writeUnsignedVarintObject(1, groupAndIndex.second);
+                b.writeEndCompound();
+                b.writeEndCompound();
+                return;
+            }
         }
     }
 
@@ -412,7 +428,11 @@ void PythonSerializationContext::serializeMutuallyRecursiveTypeGroup(MutuallyRec
         }
 
         // group hash
-        b.writeStringObject(1, group->hash().digestAsString());
+        if (shouldSerializeHashSequence()) {
+            b.writeUnsignedVarintObject(1, b.getGroupCounter(group));
+        } else {
+            b.writeStringObject(1, group->hash().digestAsString());
+        }
 
         // check if any element of this group is named. If so, we can just write the name and
         // then grab the group on the other side
@@ -609,8 +629,6 @@ void PythonSerializationContext::serializeMutuallyRecursiveTypeGroup(MutuallyRec
                             // so we just want to serialize the type of the inner
                             // which should be another MRTG reference
                             serializeNativeType(f->getTarget(), b, index);
-                        } else {
-                            throw std::runtime_error("Can't serialize empty forward " + f->name());
                         }
                     } else {
                         serializeNativeTypeInner(toSerialize, b, index);
@@ -873,18 +891,18 @@ void PythonSerializationContext::serializeNativeTypeInner(
 }
 
 void PythonSerializationContext::serializeClassMembers(
-    const std::vector<std::tuple<std::string, Type*, Instance> >& members,
+    const std::vector<MemberDefinition>& members,
     SerializationBuffer& b,
     int fieldNumber
 ) const {
     b.writeBeginCompound(fieldNumber);
     for (long k = 0; k < members.size(); k++) {
         b.writeBeginCompound(k);
-            b.writeStringObject(0, std::get<0>(members[k]));
+            b.writeStringObject(0, members[k].getName());
 
-            serializeNativeType(std::get<1>(members[k]), b, 1);
+            serializeNativeType(members[k].getType(), b, 1);
 
-            Instance defaultValue = std::get<2>(members[k]);
+            Instance defaultValue = members[k].getDefaultValue();
 
             if (defaultValue.type()->getTypeCategory() != Type::TypeCategory::catNone) {
                 b.writeBeginCompound(2);
@@ -896,6 +914,8 @@ void PythonSerializationContext::serializeClassMembers(
 
                 b.writeEndCompound();
             }
+
+            b.writeUnsignedVarintObject(3, members[k].getIsNonempty() ? 1 : 0);
         b.writeEndCompound();
     }
     b.writeEndCompound();
@@ -909,9 +929,9 @@ void PythonSerializationContext::serializeAlternativeMembers(
     b.writeBeginCompound(fieldNumber);
     for (long k = 0; k < members.size(); k++) {
         b.writeBeginCompound(k);
-            b.writeStringObject(0, std::get<0>(members[k]));
+            b.writeStringObject(0, members[k].first);
 
-            serializeNativeType(std::get<1>(members[k]), b, 1);
+            serializeNativeType(members[k].second, b, 1);
         b.writeEndCompound();
     }
     b.writeEndCompound();
