@@ -257,6 +257,18 @@ public:
     void serialize(instance_ptr self, buf_t& buffer, size_t fieldNumber) {
         buffer.writeRegisterType(fieldNumber, *(T*)self);
     }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        return 0;
+    }
+
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        copy_constructor(dest, src);
+    }
 };
 
 
