@@ -16,9 +16,10 @@ from typed_python import _types
 
 from typed_python.python_ast import (
     convertFunctionToAlgebraicPyAst,
-    cacheAstForCode
+    cacheAstForCode,
 )
 from typed_python.hash import sha_hash
+from typed_python.compiler.runtime_lock import runtimeLock
 from typed_python.type_function import ConcreteTypeFunction, reconstructTypeFunctionType, isTypeFunctionType
 from types import FunctionType, ModuleType, CodeType, MethodType, BuiltinFunctionType
 from _thread import LockType, RLock
@@ -26,19 +27,12 @@ import abc
 import numpy
 import sys
 import datetime
-import lz4.frame
 import importlib
 import threading
 import types
 import traceback
 import logging
 
-# a lock to guard the calls to importlib below. It we don't have this,
-# then two threads trying to deserialize at the same time can conflict
-# with each other, because one may see the half-imported module from
-# the other.
-_importlibLock = threading.Lock()
-
 
 _badModuleCache = set()
 
@@ -126,7 +120,8 @@ class SerializationContext:
         encodeLineInformationForCode=True,
         objectToNameOverride=None,
         internalizeTypeGroups=True,
-        serializeFunctionGlobalsAsIs=False
+        serializeFunctionGlobalsAsIs=False,
+        serializeHashSequence=False
     ):
         super().__init__()
 
@@ -140,6 +135,7 @@ class SerializationContext:
         self.encodeLineInformationForCode = encodeLineInformationForCode
         self.internalizeTypeGroups = internalizeTypeGroups
         self.serializeFunctionGlobalsAsIs = serializeFunctionGlobalsAsIs
+        self.serializeHashSequence = serializeHashSequence
 
     def addNamedObject(self, name, obj):
         self.nameToObjectOverride[name] = obj
@@ -151,19 +147,6 @@ class SerializationContext:
         del self.nameToObjectOverride[name]
         del self.objectToNameOverride[objId]
 
-    def compress(self, bytes):
-        if self.compressionEnabled:
-            res = lz4.frame.compress(bytes)
-            return res
-        else:
-            return bytes
-
-    def decompress(self, bytes):
-        if self.compressionEnabled:
-            return lz4.frame.decompress(bytes)
-        else:
-            return bytes
-
     def withFunctionGlobalsAsIs(self):
         """When serializing a function, don't replace its globals dict.
 
@@ -186,7 +169,8 @@ class SerializationContext:
             encodeLineInformationForCode=self.encodeLineInformationForCode,
             objectToNameOverride=self.objectToNameOverride,
             internalizeTypeGroups=self.internalizeTypeGroups,
-            serializeFunctionGlobalsAsIs=True
+            serializeFunctionGlobalsAsIs=True,
+            serializeHashSequence=self.serializeHashSequence
         )
 
     def withoutInternalizingTypeGroups(self):
@@ -204,7 +188,8 @@ class SerializationContext:
             encodeLineInformationForCode=self.encodeLineInformationForCode,
             objectToNameOverride=self.objectToNameOverride,
             internalizeTypeGroups=False,
-            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs
+            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs,
+            serializeHashSequence=self.serializeHashSequence
         )
 
     def withoutLineInfoEncoded(self):
@@ -217,7 +202,8 @@ class SerializationContext:
             encodeLineInformationForCode=False,
             objectToNameOverride=self.objectToNameOverride,
             internalizeTypeGroups=self.internalizeTypeGroups,
-            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs
+            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs,
+            serializeHashSequence=self.serializeHashSequence
         )
 
     def withoutCompression(self):
@@ -230,7 +216,8 @@ class SerializationContext:
             encodeLineInformationForCode=self.encodeLineInformationForCode,
             objectToNameOverride=self.objectToNameOverride,
             internalizeTypeGroups=self.internalizeTypeGroups,
-            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs
+            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs,
+            serializeHashSequence=self.serializeHashSequence
         )
 
     def withCompression(self):
@@ -243,7 +230,22 @@ class SerializationContext:
             encodeLineInformationForCode=self.encodeLineInformationForCode,
             objectToNameOverride=self.objectToNameOverride,
             internalizeTypeGroups=self.internalizeTypeGroups,
-            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs
+            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs,
+            serializeHashSequence=self.serializeHashSequence
+        )
+
+    def withSerializeHashSequence(self):
+        if self.serializeHashSequence:
+            return self
+
+        return SerializationContext(
+            nameToObjectOverride=self.nameToObjectOverride,
+            compressionEnabled=self.compressionEnabled,
+            encodeLineInformationForCode=self.encodeLineInformationForCode,
+            objectToNameOverride=self.objectToNameOverride,
+            internalizeTypeGroups=self.internalizeTypeGroups,
+            serializeFunctionGlobalsAsIs=self.serializeFunctionGlobalsAsIs,
+            serializeHashSequence=True
         )
 
     def nameForObject(self, t):
@@ -355,13 +357,13 @@ class SerializationContext:
             moduleName = ".".join(items[:-2])
 
             try:
-                if moduleName in sys.modules:
-                    module = sys.modules[moduleName]
-                else:
-                    if moduleName in _badModuleCache or not allowImport:
-                        return None
+                with runtimeLock:
+                    if moduleName in sys.modules:
+                        module = sys.modules[moduleName]
+                    else:
+                        if moduleName in _badModuleCache or not allowImport:
+                            return None
 
-                    with _importlibLock:
                         module = importlib.import_module(moduleName)
             except ModuleNotFoundError:
                 _badModuleCache.add(moduleName)
@@ -388,16 +390,16 @@ class SerializationContext:
 
         if name.startswith(".modules."):
             try:
-                if name[9:] in _badModuleCache:
-                    return None
+                with runtimeLock:
+                    if name[9:] in _badModuleCache:
+                        return None
 
-                if name[9:] in sys.modules:
-                    return sys.modules[name[9:]]
+                    if name[9:] in sys.modules:
+                        return sys.modules[name[9:]]
 
-                if not allowImport:
-                    return None
+                    if not allowImport:
+                        return None
 
-                with _importlibLock:
                     return importlib.import_module(name[9:])
             except ModuleNotFoundError:
                 _badModuleCache.add(name[9:])
@@ -418,13 +420,13 @@ class SerializationContext:
             if moduleName in _badModuleCache:
                 return None
 
-            if moduleName in sys.modules:
-                module = sys.modules[moduleName]
-            else:
-                if not allowImport:
-                    return None
+            with runtimeLock:
+                if moduleName in sys.modules:
+                    module = sys.modules[moduleName]
+                else:
+                    if not allowImport:
+                        return None
 
-                with _importlibLock:
                     module = importlib.import_module(moduleName)
 
             return getattr(module, objName, None)
@@ -616,7 +618,7 @@ class SerializationContext:
                 # clients are allowed to put references to submodules (e.g. "lxml.etree")
                 # indicating that if we use 'lxml' we also need to import 'lxml.etree'
                 representation['globals'] = {
-                    k: v for k, v in inst.__globals__.items() if k.split(".")[0] in all_names
+                    k: v for k, v in sorted(inst.__globals__.items()) if k.split(".")[0] in all_names
                 }
 
             args = (
