@@ -34,6 +34,8 @@ public:
         }
     };
 
+    typedef layout* layout_ptr;
+
     StringType() : Type(TypeCategory::catString)
     {
         m_name = "str";
@@ -125,6 +127,77 @@ public:
 
     static StringType* Make() { static StringType* res = new StringType(); return res; }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        layout_ptr& destLayout = *(layout**)dest;
+        layout_ptr& srcLayout = *(layout**)src;
+
+        if (!srcLayout) {
+            destLayout = srcLayout;
+            return;
+        }
+
+        if (srcLayout->refcount == 1) {
+            int64_t new_byteCount = srcLayout->pointcount * srcLayout->bytes_per_codepoint;
+
+            destLayout = (layout_ptr)context.slab->allocate(sizeof(layout) + new_byteCount, this);
+            destLayout->refcount = 1;
+            destLayout->hash_cache = srcLayout->hash_cache;
+            destLayout->pointcount = srcLayout->pointcount;
+            destLayout->bytes_per_codepoint = srcLayout->bytes_per_codepoint;
+
+            memcpy(destLayout->data, srcLayout->data, new_byteCount);
+            return;
+        }
+
+        auto it = context.alreadyAllocated.find((instance_ptr)srcLayout);
+        if (it == context.alreadyAllocated.end()) {
+            int64_t new_byteCount = srcLayout->pointcount * srcLayout->bytes_per_codepoint;
+
+            destLayout = (layout_ptr)context.slab->allocate(sizeof(layout) + new_byteCount, this);
+            destLayout->refcount = 0;
+            destLayout->hash_cache = srcLayout->hash_cache;
+            destLayout->pointcount = srcLayout->pointcount;
+            destLayout->bytes_per_codepoint = srcLayout->bytes_per_codepoint;
+
+            memcpy(destLayout->data, srcLayout->data, new_byteCount);
+
+            context.alreadyAllocated[(instance_ptr)srcLayout] = (instance_ptr)destLayout;
+        } else {
+            destLayout = (layout_ptr)it->second;
+        }
+
+        destLayout->refcount++;
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        layout* l = *(layout**)instance;
+
+        if (!l) {
+            return 0;
+        }
+
+        // we don't have to check if we've seen this before if the refcount is 1
+        // because there is only one holder.
+        if (l->refcount != 1) {
+            if (alreadyVisited.find((void*)l) != alreadyVisited.end()) {
+                return 0;
+            }
+
+            alreadyVisited.insert((void*)l);
+        }
+
+        if (outSlabs && Slab::slabForAlloc(l)) {
+            outSlabs->insert(Slab::slabForAlloc(l));
+            return 0;
+        }
+
+        return bytesRequiredForAllocation(l->pointcount * l->bytes_per_codepoint + sizeof(layout));
+    }
+
     template<class buf_t>
     void serialize(instance_ptr self, buf_t& buffer, size_t fieldNumber) {
         if (bytes_per_codepoint(self) == 1) {
