@@ -224,8 +224,8 @@ void TupleOrListOfType::destroy(instance_ptr selfPtr) {
 
     if (self->refcount.fetch_sub(1) == 1) {
         m_element_type->destroy(self->count, [&](int64_t k) {return eltPtr(self,k);});
-        free(self->data);
-        free(self);
+        tp_free(self->data);
+        tp_free(self);
     }
 }
 
@@ -264,7 +264,11 @@ void TupleOrListOfType::reserve(instance_ptr self, size_t target) {
         target = self_layout->count;
     }
 
-    self_layout->data = (uint8_t*)realloc(self_layout->data, getEltType()->bytecount() * target);
+    self_layout->data = (uint8_t*)tp_realloc(
+        self_layout->data,
+        getEltType()->bytecount() * self_layout->reserved,
+        getEltType()->bytecount() * target
+    );
     self_layout->reserved = target;
 }
 
@@ -275,7 +279,7 @@ void TupleOrListOfType::reverse(instance_ptr self) {
     long elt_size = getEltType()->bytecount();
 
     // swap memory without caring what the elements are
-    uint8_t* swap = (uint8_t*)malloc(elt_size);
+    uint8_t* swap = (uint8_t*)tp_malloc(elt_size);
     for (long k = 0; k < self_layout->count / 2; k++) {
         uint8_t* ptr1 = (uint8_t*)eltPtr(self_layout, k);
         uint8_t* ptr2 = (uint8_t*)eltPtr(self_layout, self_layout->count - 1 - k);
@@ -283,7 +287,7 @@ void TupleOrListOfType::reverse(instance_ptr self) {
         memcpy(ptr1, ptr2, elt_size);
         memcpy(ptr2, swap, elt_size);
     }
-    free(swap);
+    tp_free(swap);
 }
 
 //static
@@ -303,7 +307,8 @@ void ListOfType::append(instance_ptr self, instance_ptr other) {
     layout_ptr& self_layout = *(layout_ptr*)self;
 
     if (!self_layout) {
-        self_layout = (layout_ptr)malloc(sizeof(layout) + getEltType()->bytecount() * 1);
+        self_layout = (layout_ptr)tp_malloc(sizeof(layout));
+        self_layout->data = (uint8_t*)tp_malloc(getEltType()->bytecount());
 
         self_layout->count = 1;
         self_layout->refcount = 1;
@@ -314,7 +319,11 @@ void ListOfType::append(instance_ptr self, instance_ptr other) {
     } else {
         if (self_layout->count == self_layout->reserved) {
             int64_t new_reserved = self_layout->reserved * 1.25 + 1;
-            self_layout->data = (uint8_t*)realloc(self_layout->data, getEltType()->bytecount() * new_reserved);
+            self_layout->data = (uint8_t*)tp_realloc(
+                self_layout->data,
+                getEltType()->bytecount() * self_layout->reserved,
+                getEltType()->bytecount() * new_reserved
+            );
             self_layout->reserved = new_reserved;
         }
 
