@@ -16,11 +16,13 @@
 
 #pragma once
 
+#include "Memory.hpp"
 #include <Python.h>
 #include <string>
 #include <vector>
 #include <algorithm>
 #include <functional>
+#include <unordered_set>
 #include <mutex>
 #include <set>
 #include <map>
@@ -35,6 +37,8 @@
 #include "TypeOrPyobj.hpp"
 #include "util.hpp"
 #include "MutuallyRecursiveTypeGroup.hpp"
+#include "Slab.hpp"
+#include "DeepcopyContext.hpp"
 
 class SerializationBuffer;
 class DeserializationBuffer;
@@ -143,6 +147,14 @@ public:
         return m_typeCategory == catTuple;
     }
 
+    bool isString() const {
+        return m_typeCategory == catString;
+    }
+
+    bool isBytes() const {
+        return m_typeCategory == catBytes;
+    }
+
     bool isListOf() const {
         return m_typeCategory == catListOf;
     }
@@ -159,6 +171,10 @@ public:
         return m_typeCategory == catDict;
     }
 
+    bool isSet() const {
+        return m_typeCategory == catSet;
+    }
+
     bool isConstDict() const {
         return m_typeCategory == catConstDict;
     }
@@ -175,6 +191,10 @@ public:
         return m_typeCategory == catFunction;
     }
 
+    bool isPythonObjectOfType() const {
+        return m_typeCategory == catPythonObjectOfType;
+    }
+
     bool isClass() const {
         return m_typeCategory == catClass;
     }
@@ -191,6 +211,10 @@ public:
         return m_typeCategory == catConcreteAlternative;
     }
 
+    bool isTypedCell() const {
+        return m_typeCategory == catTypedCell;
+    }
+
     bool isForward() const {
         return m_typeCategory == catForward;
     }
@@ -304,6 +328,46 @@ public:
 
     typed_python_hash_type hash(instance_ptr left);
 
+    void deepcopy(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    );
+
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        throw std::runtime_error(
+            "deepcopyConcrete not implemented for " + name() +
+            " of category " + categoryToString(getTypeCategory())
+        );
+    }
+
+    // compute the allocated bytecount of an object, including python objects (but not walking
+    // into types, modules, or functions). Don't include storage for 'instance' itself - just storage
+    // that's allocated on the heap by this object. If 'outSlabs' is not the nullpointer, then if we
+    // hit an allocation that's mapped to a slab, just mark the slab and return.
+    size_t deepBytecount(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        assertForwardsResolvedSufficientlyToInstantiate();
+
+        return this->check([&](auto& subtype) {
+            return subtype.deepBytecountConcrete(instance, alreadyVisited, outSlabs);
+        });
+    }
+
+    size_t deepBytecountConcrete(
+        instance_ptr instance,
+        std::unordered_set<void*>& alreadyVisited,
+        std::set<Slab*>* outSlabs
+    ) {
+        throw std::runtime_error(
+            "deepBytecountConcrete not implemented for " + name() +
+            " of category " + categoryToString(getTypeCategory())
+        );
+    }
+
     template<class buf_t>
     void serialize(instance_ptr left, buf_t& buffer, size_t fieldNumber) {
         assertForwardsResolvedSufficientlyToInstantiate();
