@@ -20,8 +20,7 @@ from typed_python.test_util import currentMemUsageMb
 from typed_python._types import is_default_constructible
 from typed_python import (
     Int16, UInt64, Float32, ListOf, TupleOf, OneOf, NamedTuple, Class, Alternative,
-    ConstDict, Member, _types, Forward, Final, Function, Entrypoint, Tuple,
-    Held, RefTo, refTo, pointerTo, copy
+    ConstDict, Member, _types, Forward, Final, Function, Entrypoint, Tuple
 )
 
 
@@ -1486,92 +1485,17 @@ class NativeClassTypesTests(unittest.TestCase):
         check()
         Entrypoint(check)()
 
-    def test_class_held_pointers(self):
-        class C(Class):
-            x = Member(int)
-            y = Member(float)
+    def test_class_delitem(self):
+        class C(Class, Final):
+            deleted = Member(int)
 
-        aC = C(x=10, y=20.5)
-
-        cPtr = pointerTo(aC)
-
-        self.assertEqual(cPtr.x.get(), 10)
-        self.assertEqual(cPtr.y.get(), 20.5)
-
-        cPtr.x.set(15)
-
-        self.assertEqual(cPtr.x.get(), 15)
-        self.assertEqual(aC.x, 15)
-
-    def test_held_class_instances(self):
-        class H(Class, Final):
-            y = Member(int)
-
-            def f(self, x):
-                return self.y + x
-
-        class C(Class):
-            x = Member(Held(H))
-
-        self.assertEqual(_types.bytecount(C), 8)
-
-        # 8 bytes for the float, 1 for 'C''s initialize field, and 1 for H's
-        # eventually we should be able to build classes where we know the
-        # values are always initialized.
-        self.assertEqual(_types.bytecount(Held(C)), 10)
+            def __delitem__(self, k):
+                self.deleted = k
 
         c = C()
-        self.assertTrue(isinstance(c.x, RefTo(Held(H))))
+        del c[10]
 
-        self.assertEqual(c.x.y, 0)
-        c.x.y = 20
-        self.assertEqual(c.x.y, 20)
-
-        self.assertEqual(c.x.f(30), 50)
-
-        c2 = C()
-        c2.x.y = 10
-
-        # we should be able to copy from one C's x to another
-        c.x = c2.x
-
-        # and then change c2.x, which is held inside of 'c'.
-        c2.x.y = 30
-
-        self.assertEqual(c.x.y, 10)
-        self.assertEqual(c2.x.y, 30)
-
-        refTo(c).x = refTo(c2).x
-        self.assertEqual(c.x.y, 30)
-
-    def test_list_of_held(self):
-        class H(Class, Final):
-            x = Member(int)
-            y = Member(int)
-
-        t = ListOf(Held(H))()
-
-        t.resize(10)
-
-        self.assertTrue(isinstance(t[2], RefTo(Held(H))))
-
-        t[2].x = 20
-        self.assertEqual(t[2].x, 20)
-
-        # copy the reference out to the heap - it's now a
-        # separate object with its own refcount
-        elt2Copy = copy(t[2])
-        self.assertEqual(elt2Copy.x, 20)
-
-        # and changing it doesn't change the list's value
-        elt2Copy.x = 30
-        self.assertEqual(elt2Copy.x, 30)
-        self.assertEqual(t[2].x, 20)
-
-        # copying held classes around makes sense, and doesn't
-        # duplicate their internals.
-        t[3] = t[2]
-        self.assertEqual(t[3].x, 20)
+        assert c.deleted == 10
 
     def test_class_len_leak(self):
         class C(Class, Final):
@@ -1627,14 +1551,82 @@ class NativeClassTypesTests(unittest.TestCase):
         with self.assertRaisesRegex(Exception, "Cannot construct the value None from an instance of bool"):
             c[10] = 20
 
-    def test_cant_create_held_classes_in_interpreter(self):
+    def test_class_delattr(self):
         class C(Class, Final):
-            x = Member(int)
+            x = Member(str)
 
-        H = Held(C)
+        c = C(x="hi")
 
-        with self.assertRaises(Exception):
-            H()
+        assert c.x == "hi"
 
-        with self.assertRaises(Exception):
-            H(x=1)
+        del c.x
+
+        with self.assertRaises(AttributeError):
+            c.x
+
+        assert not hasattr(c, 'x')
+
+        c.x = "bye"
+
+        assert c.x == "bye"
+
+    def test_class_nonempty(self):
+        class CNonempty(Class, Final):
+            x = Member(str, nonempty=True)
+
+            def __init__(self):
+                pass
+
+        class CEmpty(Class, Final):
+            x = Member(str)
+
+            def __init__(self):
+                pass
+
+        assert CNonempty().x == ""
+
+        cN = CNonempty()
+        cE = CEmpty()
+
+        with self.assertRaisesRegex(Exception, "Attribute 'x' cannot be deleted"):
+            del cN.x
+
+        assert hasattr(cE, 'x')
+        del cE.x
+
+        with self.assertRaisesRegex(Exception, "Attribute 'x' is not initialized"):
+            del cE.x
+
+        assert not hasattr(cE, 'x')
+        cE.x = "hihi"
+        assert hasattr(cE, 'x')
+
+    def test_class_nonempty_forces_construction(self):
+        class SomeOtherClass(Class):
+            pass
+
+        class CNonempty(Class, Final):
+            x = Member(SomeOtherClass, nonempty=True)
+
+            def __init__(self):
+                pass
+
+        class CEmpty(Class, Final):
+            x = Member(SomeOtherClass)
+
+            def __init__(self):
+                pass
+
+        assert hasattr(CNonempty(), 'x')
+        assert not hasattr(CEmpty(), 'x')
+
+    def test_class_mro_index_is_zero_for_self(self):
+        class BaseClass(Class):
+            pass
+
+        class ChildClass(BaseClass, Final):
+            pass
+
+        assert _types.classGetDispatchIndex(BaseClass, ChildClass) == 1
+        assert _types.classGetDispatchIndex(BaseClass, BaseClass) == 0
+        assert _types.classGetDispatchIndex(ChildClass, ChildClass) == 0
