@@ -16,6 +16,7 @@ from typed_python.compiler.global_variable_definition import GlobalVariableMetad
 import typed_python.compiler
 import typed_python.compiler.native_ast as native_ast
 from typed_python.compiler.type_wrappers.wrapper import Wrapper
+from typed_python.compiler.merge_type_wrappers import mergeTypeWrappers
 import typed_python.compiler.type_wrappers.runtime_functions as runtime_functions
 import types
 
@@ -24,7 +25,6 @@ from typed_python.compiler.conversion_level import ConversionLevel
 from typed_python.SerializationContext import SerializationContext
 from typed_python.internals import makeFunctionType, FunctionOverload
 from typed_python.compiler.function_stack_state import FunctionStackState
-from typed_python.compiler.type_wrappers.one_of_wrapper import OneOfWrapper
 from typed_python.compiler.python_object_representation import pythonObjectRepresentation
 from typed_python.compiler.python_object_representation import pythonObjectRepresentationType
 from typed_python.compiler.typed_expression import TypedExpression
@@ -43,13 +43,9 @@ builtinValueIdToNameAndValue = {id(v): (k, v) for k, v in __builtins__.items()}
 
 typeWrapper = lambda t: typed_python.compiler.python_object_representation.typedPythonTypeToTypeWrapper(t)
 
-ExpressionIntermediate = Alternative(
-    "ExpressionIntermediate",
-    Effect={"expr": native_ast.Expression},
-    Terminal={"expr": native_ast.Expression},
-    Simple={"name": str, "expr": native_ast.Expression},
-    StackSlot={"name": str, "expr": native_ast.Expression}
-)
+
+ExpressionIntermediate = native_ast.ExpressionIntermediate
+
 
 _pyFuncToFuncCache = {}
 
@@ -85,7 +81,12 @@ class ExpressionConversionContext:
         return not self.intermediates
 
     def inputArg(self, type, name):
-        return TypedExpression(self, native_ast.Expression.Variable(name), type, type.is_pass_by_ref)
+        return TypedExpression(
+            self,
+            native_ast.Expression.Variable(name) if not type.is_empty else native_ast.nullExpr,
+            type,
+            type.is_pass_by_ref
+        )
 
     def zero(self, T):
         """Return a TypedExpression matching the Zero form of the native layout of type T.
@@ -741,13 +742,8 @@ class ExpressionConversionContext:
             assert isinstance(expr, TypedExpression), type(expr)
             expr = expr.nonref_expr
 
-        for i in reversed(self.intermediates):
-            if i.matches.Terminal or i.matches.Effect:
-                expr = i.expr >> expr
-            elif i.matches.StackSlot:
-                expr = i.expr >> expr
-            elif i.matches.Simple:
-                expr = native_ast.Expression.Let(var=i.name, val=i.expr, within=expr)
+        if len(self.intermediates):
+            expr = native_ast.Expression.ApplyIntermediates(base=expr, intermediates=self.intermediates)
 
         if self.teardowns:
             expr = native_ast.Expression.Finally(expr=expr, teardowns=self.teardowns)
@@ -1217,12 +1213,21 @@ class ExpressionConversionContext:
 
             return
 
+        constantRetval = call_target.functionMetadata.getConstantReturnValue()
+
         if call_target.output_type.is_pass_by_ref:
             assert len(call_target.named_call_target.arg_types) == len(native_args) + 1, "\n\n%s\n%s" % (
                 call_target.named_call_target.arg_types,
                 [a.expr_type for a in args]
             )
 
+            if constantRetval is not None:
+                self.push(
+                    call_target.output_type,
+                    lambda output_slot: call_target.call(output_slot.expr, *native_args)
+                )
+                return self.constant(constantRetval)
+
             return self.push(
                 call_target.output_type,
                 lambda output_slot: call_target.call(output_slot.expr, *native_args)
@@ -1235,6 +1240,10 @@ class ExpressionConversionContext:
                     f"{len(call_target.named_call_target.arg_types)} and we got {len(native_args)}"
                 )
 
+            if constantRetval is not None:
+                self.pushEffect(call_target.call(*native_args))
+                return self.constant(constantRetval)
+
             return self.pushPod(
                 call_target.output_type,
                 call_target.call(*native_args)
@@ -1381,15 +1390,24 @@ class ExpressionConversionContext:
         if varType is not None and varType != expr.expr_type.typeRepresentation:
             if expr.expr_type.is_oneof_wrapper:
                 # if it's a concrete type, then it must be one of our subtypes
-                if isinstance(varType, type) and not issubclass(varType, OneOf):
-                    return expr.expr_type.refAsType(expr.context, expr, varType)
-                else:
-                    # it's possible that it's a subset of the oneof types
-                    return expr.convert_to_type(varType, ConversionLevel.Signature)
+                if varType in expr.expr_type.typeRepresentation.Types:
+                    # this is only true if we know for certain that exactly one of these
+                    # types can be cast to varType and that's it
+                    index = expr.expr_type.unambiguousTypeIndexFor(varType)
+                    if index is not None:
+                        return expr.refAs(index)
+
+                # it's possible that it's a subset of the oneof types
+                return expr.convert_to_type(varType, ConversionLevel.Signature, assumeSuccessful=True)
 
             if getattr(expr.expr_type.typeRepresentation, "__typed_python_category__", None) == "Alternative" and \
                     getattr(varType, "__typed_python_category__", None) == "ConcreteAlternative":
                 return expr.changeType(varType)
+
+            # if its a class, and the variable is a subclass of our type
+            if expr.expr_type.is_class_wrapper and issubclass(varType, expr.expr_type.classType):
+                return expr.convert_to_type(typeWrapper(varType), ConversionLevel.Signature, assumeSuccessful=True)
+
         return expr
 
     def namedVariableLookup(self, name):
@@ -1535,7 +1553,7 @@ class ExpressionConversionContext:
 
             # this is the merge of all the possible types that could
             # come out of the subexpression.
-            return_type = OneOfWrapper.mergeTypes(resultTypes)
+            return_type = mergeTypeWrappers(resultTypes)
 
             # allocate a new slot for the final value. every pathway
             # will either write into it and intialize it, or throw an
@@ -1829,12 +1847,7 @@ class ExpressionConversionContext:
                 elif false_res is None:
                     out_type = true_res.expr_type
                 elif true_res.expr_type != false_res.expr_type:
-                    out_type = typeWrapper(
-                        OneOf(
-                            true_res.expr_type.interpreterTypeRepresentation,
-                            false_res.expr_type.interpreterTypeRepresentation
-                        )
-                    )
+                    out_type = mergeTypeWrappers([true_res.expr_type, false_res.expr_type])
                 else:
                     out_type = true_res.expr_type
 
@@ -1979,6 +1992,9 @@ class ExpressionConversionContext:
             ).convert_call([], {})
 
         if ast.matches.Constant:
+            if ast.value is isinstance:
+                return pythonObjectRepresentation(self, ast.value)
+
             return self.constant(ast.value, allowArbitrary=True)
 
         raise ConversionException("can't handle python expression type %s" % ast.Name)
