@@ -25,13 +25,14 @@ from typed_python.compiler.type_wrappers.method_descriptor_wrapper import Method
 from typed_python.compiler.type_wrappers.python_type_object_wrapper import PythonTypeObjectWrapper
 from typed_python.compiler.type_wrappers.module_wrapper import ModuleWrapper
 from typed_python.compiler.type_wrappers.typed_cell_wrapper import TypedCellWrapper
+from typed_python.compiler.type_wrappers.unresolved_forward_type_wrapper import UnresolvedForwardTypeWrapper
 from typed_python.compiler.type_wrappers.python_free_function_wrapper import PythonFreeFunctionWrapper
 from typed_python.compiler.type_wrappers.python_free_object_wrapper import PythonFreeObjectWrapper
 from typed_python.compiler.type_wrappers.python_typed_function_wrapper import PythonTypedFunctionWrapper
 from typed_python.compiler.type_wrappers.value_wrapper import ValueWrapper
 from typed_python.compiler.type_wrappers.tuple_of_wrapper import TupleOfWrapper
 from typed_python.compiler.type_wrappers.pointer_to_wrapper import PointerToWrapper, PointerToObjectWrapper
-from typed_python.compiler.type_wrappers.ref_to_wrapper import RefToWrapper
+from typed_python.compiler.type_wrappers.ref_to_wrapper import RefToWrapper, RefToObjectWrapper
 from typed_python.compiler.type_wrappers.list_of_wrapper import ListOfWrapper
 from typed_python.compiler.type_wrappers.isinstance_wrapper import IsinstanceWrapper
 from typed_python.compiler.type_wrappers.issubclass_wrapper import IssubclassWrapper
@@ -67,14 +68,16 @@ from typed_python.compiler.type_wrappers.abs_wrapper import AbsWrapper
 from typed_python.compiler.type_wrappers.min_max_wrapper import MinWrapper, MaxWrapper
 from typed_python.compiler.type_wrappers.repr_wrapper import ReprWrapper
 from types import ModuleType
-from typed_python._types import TypeFor, bytecount, prepareArgumentToBePassedToCompiler
+from typed_python._types import (
+    TypeFor, bytecount, prepareArgumentToBePassedToCompiler, allForwardTypesResolved
+)
 from typed_python import (
     Type, Int32, Int16, Int8, UInt64, UInt32, UInt16,
     UInt8, Float32, makeNamedTuple,
     ListOf, isCompiled,
     typeKnownToCompiler,
     localVariableTypesKnownToCompiler,
-    pointerTo
+    pointerTo, refTo
 )
 
 # the type of bound C methods on types.
@@ -139,6 +142,9 @@ def _typedPythonTypeToTypeWrapper(t):
 
     assert isinstance(t, type), t
 
+    if not allForwardTypesResolved(t):
+        return UnresolvedForwardTypeWrapper(t)
+
     if not hasattr(t, '__typed_python_category__'):
         # this is will be a PythonObjectOfType, but we never actually return such an object
         # to the interpreter
@@ -268,6 +274,9 @@ def pythonObjectRepresentation(context, f, owningGlobalScopeAndName=None):
     if f is pointerTo:
         return TypedExpression(context, native_ast.nullExpr, PointerToObjectWrapper(), False)
 
+    if f is refTo:
+        return TypedExpression(context, native_ast.nullExpr, RefToObjectWrapper(), False)
+
     if f is isCompiled:
         return TypedExpression(context, native_ast.nullExpr, IsCompiledWrapper(), False)
 
