@@ -18,8 +18,24 @@ from math import trunc, floor, ceil
 import unittest
 from flaky import flaky
 from typed_python import (
-    Class, Dict, ConstDict, TupleOf, ListOf, Member, OneOf, UInt64, Int16,
-    Float32, Final, makeNamedTuple, Compiled, Function, Held, Value, pointerTo
+    Class,
+    Dict,
+    ConstDict,
+    TupleOf,
+    ListOf,
+    Member,
+    OneOf,
+    UInt64,
+    Int16,
+    Float32,
+    Final,
+    makeNamedTuple,
+    Compiled,
+    Function,
+    Value,
+    pointerTo,
+    refTo,
+    Forward
 )
 import typed_python._types as _types
 from typed_python.compiler.runtime import Entrypoint, Runtime, CountCompilationsVisitor
@@ -264,7 +280,7 @@ class TestClassCompilationCompilation(unittest.TestCase):
         print("Times were ", elapsed1, elapsed2)
         # they should take the same amount of time. Takes about 1 second for me to
         # tun this test. Note that we generate two entrypoints (one for float, one for int)
-        self.assertTrue(.5 <= elapsed1 / elapsed2 <= 2.0, elapsed1 / elapsed2)
+        self.assertTrue(0.5 <= elapsed1 / elapsed2 <= 2.0, elapsed1 / elapsed2)
 
     def test_compile_class_method(self):
         c = AClass(x=20)
@@ -273,10 +289,7 @@ class TestClassCompilationCompilation(unittest.TestCase):
         uncompiled_res = c.loop(1000000)
         uncompiled_time = time.time() - t0
 
-        self.assertEqual(
-            AClass.loop.resultTypeFor(AClass, int).typeRepresentation,
-            float
-        )
+        self.assertEqual(AClass.loop.resultTypeFor(AClass, int).typeRepresentation, float)
 
         t0 = time.time()
         compiled_res = c.loop(1000000)
@@ -414,6 +427,7 @@ class TestClassCompilationCompilation(unittest.TestCase):
 
         # if you override a method you can't change its output type.
         with self.assertRaisesRegex(Exception, "Overloads of 'f' don't have the same return type"):
+
             class BadChild(Base):
                 def f(self) -> float:
                     pass
@@ -437,12 +451,16 @@ class TestClassCompilationCompilation(unittest.TestCase):
         self.assertEqual(Function(lambda c, a: c.f(a)).resultTypeFor(TestClass, int).typeRepresentation, int)
         self.assertEqual(Function(lambda c, a: c.f(a)).resultTypeFor(TestClass, str).typeRepresentation, str)
         self.assertEqual(
-            set(Function(lambda c, a: c.f(a)).resultTypeFor(TestClass, OneOf(str, int)).typeRepresentation.Types),
-            set(OneOf(str, int).Types)
+            set(
+                Function(lambda c, a: c.f(a))
+                .resultTypeFor(TestClass, OneOf(str, int))
+                .typeRepresentation.Types
+            ),
+            set(OneOf(str, int).Types),
         )
         self.assertEqual(
             set(Function(lambda c, a: c.f(a)).resultTypeFor(TestClass, object).typeRepresentation.Types),
-            set(OneOf(str, int).Types)
+            set(OneOf(str, int).Types),
         )
 
         @Compiled
@@ -770,7 +788,7 @@ class TestClassCompilationCompilation(unittest.TestCase):
         elapsedNoDispatch = t2 - t1
         speedup = elapsedDispatch / elapsedNoDispatch
 
-        self.assertTrue(.7 < speedup < 1.3, speedup)
+        self.assertTrue(0.7 < speedup < 1.3, speedup)
 
     def test_class_return_self(self):
         class C(Class):
@@ -785,7 +803,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
         self.assertIsInstance(c_f(C()), C)
 
     def test_compile_class_magic_methods(self):
-
         class C(Class, Final):
             s = Member(str)
 
@@ -801,11 +818,11 @@ class TestClassCompilationCompilation(unittest.TestCase):
             __call__ = lambda self, i: "my call"
             __len__ = lambda self: 42
             __contains__ = lambda self, item: item == 1
-            __bytes__ = lambda self: b'my bytes'
+            __bytes__ = lambda self: b"my bytes"
 
             __int__ = lambda self: 43
             __float__ = lambda self: 44.44
-            __complex__ = lambda self: 3+4j
+            __complex__ = lambda self: 3 + 4j
 
             __add__ = lambda self, other: C("add")
             __sub__ = lambda self, other: C("sub")
@@ -972,11 +989,47 @@ class TestClassCompilationCompilation(unittest.TestCase):
             x **= C("")
             return x
 
-        test_cases = [f_int, f_float, f_bool, f_str, f_repr, f_call, f_0in, f_1in, f_len,
-                      f_add, f_sub, f_mul, f_div, f_floordiv, f_matmul, f_mod, f_and, f_or, f_xor, f_rshift, f_lshift, f_pow,
-                      f_neg, f_pos, f_invert, f_abs,
-                      f_iadd, f_isub, f_imul, f_idiv, f_ifloordiv, f_imatmul,
-                      f_imod, f_iand, f_ior, f_ixor, f_irshift, f_ilshift, f_ipow]
+        test_cases = [
+            f_int,
+            f_float,
+            f_bool,
+            f_str,
+            f_repr,
+            f_call,
+            f_0in,
+            f_1in,
+            f_len,
+            f_add,
+            f_sub,
+            f_mul,
+            f_div,
+            f_floordiv,
+            f_matmul,
+            f_mod,
+            f_and,
+            f_or,
+            f_xor,
+            f_rshift,
+            f_lshift,
+            f_pow,
+            f_neg,
+            f_pos,
+            f_invert,
+            f_abs,
+            f_iadd,
+            f_isub,
+            f_imul,
+            f_idiv,
+            f_ifloordiv,
+            f_imatmul,
+            f_imod,
+            f_iand,
+            f_ior,
+            f_ixor,
+            f_irshift,
+            f_ilshift,
+            f_ipow,
+        ]
 
         for f in test_cases:
             compiled_f = Compiled(f)
@@ -985,7 +1038,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
             self.assertEqual(r1, r2)
 
     def test_compile_class_reverse_methods(self):
-
         class C(Class, Final):
             s = Member(str)
             __radd__ = lambda self, other: "radd" + repr(other)
@@ -1002,8 +1054,17 @@ class TestClassCompilationCompilation(unittest.TestCase):
             __rxor__ = lambda self, other: "rxor" + repr(other)
             __ror__ = lambda self, other: "ror" + repr(other)
 
-        values = [1, Int16(1), UInt64(1), 1.234, Float32(1.234), True, "abc",
-                  ListOf(int)((1, 2)), ConstDict(str, str)({"a": "1"})]
+        values = [
+            1,
+            Int16(1),
+            UInt64(1),
+            1.234,
+            Float32(1.234),
+            True,
+            "abc",
+            ListOf(int)((1, 2)),
+            ConstDict(str, str)({"a": "1"}),
+        ]
         for v in values:
             T = type(v)
 
@@ -1046,15 +1107,27 @@ class TestClassCompilationCompilation(unittest.TestCase):
             def f_ror(v: T, x: C):
                 return v * x
 
-            for f in [f_radd, f_rsub, f_rmul, f_rmatmul, f_rtruediv, f_rfloordiv, f_rmod, f_rpow,
-                      f_rlshift, f_rrshift, f_rand, f_rxor, f_ror]:
+            for f in [
+                f_radd,
+                f_rsub,
+                f_rmul,
+                f_rmatmul,
+                f_rtruediv,
+                f_rfloordiv,
+                f_rmod,
+                f_rpow,
+                f_rlshift,
+                f_rrshift,
+                f_rand,
+                f_rxor,
+                f_ror,
+            ]:
                 r1 = f(v, C())
                 compiled_f = Compiled(f)
                 r2 = compiled_f(v, C())
                 self.assertEqual(r1, r2)
 
     def test_compile_class_format(self):
-
         class C1(Class, Final):
             pass
 
@@ -1108,7 +1181,7 @@ class TestClassCompilationCompilation(unittest.TestCase):
 
     def test_compile_class_bytes(self):
         class C(Class, Final):
-            __bytes__ = lambda self: b'my bytes'
+            __bytes__ = lambda self: b"my bytes"
 
         def f_bytes(x: C):
             return bytes(x)
@@ -1119,8 +1192,22 @@ class TestClassCompilationCompilation(unittest.TestCase):
         r2 = c_f(v)
         self.assertEqual(r1, r2)
 
-    def test_compile_class_attr(self):
+    def test_compile_class_delitem(self):
+        class C(Class, Final):
+            deleted = Member(int)
 
+            def __delitem__(self, k):
+                self.deleted = k
+
+        @Entrypoint
+        def delIt(c, k):
+            del c[k]
+
+        c = C()
+        delIt(c, 10)
+        assert c.deleted == 10
+
+    def test_compile_class_attr(self):
         class C(Class, Final):
             d = Member(Dict(str, str))
             i = Member(int)
@@ -1331,7 +1418,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
         self.assertTrue(finalMem < initMem + 2)
 
     def test_compile_class_comparison_defaults(self):
-
         class C(Class, Final):
             i = Member(int)
             s = Member(str)
@@ -1388,7 +1474,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
         self.assertTrue(isEqObj(C(x=10), makeNamedTuple(x=10)))
 
     def test_compile_class_comparison_methods(self):
-
         class C(Class, Final):
             i = Member(int)
             s = Member(str)
@@ -1451,7 +1536,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
                 self.assertEqual(r1, r2)
 
     def test_compile_class_hash_special_value(self):
-
         class C(Class, Final):
             i = Member(int)
 
@@ -1468,7 +1552,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
         self.assertEqual(c_hash(C(i=-1)), -2)
 
     def test_compile_class_getsetitem(self):
-
         class C(Class, Final):
             d = Member(Dict(int, int))
 
@@ -1503,7 +1586,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
             self.assertEqual(c_getitem(c, i), i + 200)
 
     def test_compile_class_float_conv(self):
-
         class C0(Class, Final):
             __int__ = lambda self: 123
             __float__ = lambda self: 1234.5
@@ -1601,7 +1683,9 @@ class TestClassCompilationCompilation(unittest.TestCase):
             x ^= ClassWithoutInplaceOp()
             return x
 
-        expected = ClassWithoutInplaceOp(s="start add sub mul matmul truediv floordiv mod pow lshift rshift and or xor")
+        expected = ClassWithoutInplaceOp(
+            s="start add sub mul matmul truediv floordiv mod pow lshift rshift and or xor"
+        )
         v = ClassWithoutInplaceOp(s="start")
         r1 = inplace(v)
         self.assertEqual(r1.s, expected.s)
@@ -1616,10 +1700,7 @@ class TestClassCompilationCompilation(unittest.TestCase):
             def __float__(self):
                 return 1.0
 
-        with self.assertRaisesRegex(
-            TypeError,
-            "Cannot construct a new float from an instance of C"
-        ):
+        with self.assertRaisesRegex(TypeError, "Cannot construct a new float from an instance of C"):
             aList.append(C())
 
         @Function
@@ -1748,20 +1829,20 @@ class TestClassCompilationCompilation(unittest.TestCase):
             self.compileCheck(lambda: T().f(1, y=2))
             self.compileCheck(lambda: T().f(1, y=2))
 
-            self.compileCheck(lambda: T().f('1', '2'))
-            self.compileCheck(lambda: T().f(x='1', y='2'))
-            self.compileCheck(lambda: T().f('1', y='2'))
-            self.compileCheck(lambda: T().f('1', y='2'))
+            self.compileCheck(lambda: T().f("1", "2"))
+            self.compileCheck(lambda: T().f(x="1", y="2"))
+            self.compileCheck(lambda: T().f("1", y="2"))
+            self.compileCheck(lambda: T().f("1", y="2"))
 
-            self.compileCheck(lambda: T().f('1', 2))
-            self.compileCheck(lambda: T().f(x='1', y=2))
-            self.compileCheck(lambda: T().f('1', y=2))
-            self.compileCheck(lambda: T().f('1', y=2))
+            self.compileCheck(lambda: T().f("1", 2))
+            self.compileCheck(lambda: T().f(x="1", y=2))
+            self.compileCheck(lambda: T().f("1", y=2))
+            self.compileCheck(lambda: T().f("1", y=2))
 
-            self.compileCheck(lambda: T().f(1, '2'))
-            self.compileCheck(lambda: T().f(x=1, y='2'))
-            self.compileCheck(lambda: T().f(1, y='2'))
-            self.compileCheck(lambda: T().f(1, y='2'))
+            self.compileCheck(lambda: T().f(1, "2"))
+            self.compileCheck(lambda: T().f(x=1, y="2"))
+            self.compileCheck(lambda: T().f(1, y="2"))
+            self.compileCheck(lambda: T().f(1, y="2"))
 
     def test_class_with_global_closure_variables(self):
         def makeInt():
@@ -1830,13 +1911,11 @@ class TestClassCompilationCompilation(unittest.TestCase):
         @Entrypoint
         def doit():
             res = f(Child())
-            print("done")
             return res
 
         @Entrypoint
         def done():
             doit()
-            print("done")
 
         done()
 
@@ -1893,94 +1972,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
         self.assertEqual(add(X(), 1), 1)
         self.assertEqual(add(X(), 1.5), 2)
 
-    def test_compile_held_class(self):
-        class H(Class, Final):
-            x = Member(int)
-            y = Member(float)
-
-            def f(self):
-                return self.x + self.y
-
-            def typeOfSelf(self):
-                return type(self)
-
-        class C(Class, Final):
-            h1 = Member(Held(H))
-            h2 = Member(Held(H))
-
-        c = C()
-
-        def check(x):
-            self.assertEqual(Entrypoint(x)(c), x(c))
-
-        check(lambda c: type(c.h1))
-        check(lambda c: c.h1)
-        check(lambda c: type(c.h1.x))
-        check(lambda c: c.h1.x)
-
-        c.h1.x = 20
-        check(lambda c: c.h1.x)
-
-        def move(c):
-            c.h2 = c.h1
-            return c.h2.x
-
-        check(move)
-
-        check(lambda c: c.h1.f())
-        check(lambda c: c.h1.typeOfSelf())
-
-    def test_compile_list_of_held_class(self):
-        class H(Class, Final):
-            x = Member(int)
-            y = Member(float)
-
-            def f(self):
-                return self.x + self.y
-
-            def increment(self):
-                self.x += 1
-                self.y += 1
-
-        aList = ListOf(Held(H))()
-
-        @Entrypoint
-        def resizeList(l):
-            l.resize(10)
-
-        resizeList(aList)
-
-        self.assertEqual(len(aList), 10)
-        self.assertEqual(aList[0].x, 0)
-        self.assertEqual(aList[0].y, 0.0)
-
-        @Entrypoint
-        def getitem(l, x):
-            return l[x]
-
-        self.assertEqual(aList[0], getitem(aList, 0))
-
-        @Entrypoint
-        def incrementAllRange(l):
-            for i in range(len(l)):
-                ref = l[i]
-                ref.increment()
-
-        incrementAllRange(aList)
-
-        self.assertEqual(getitem(aList, 0).x, 1)
-        self.assertEqual(getitem(aList, 5).x, 1)
-
-        @Entrypoint
-        def incrementViaIterator(l):
-            for i in l:
-                i.increment()
-
-        incrementViaIterator(aList)
-
-        self.assertEqual(getitem(aList, 0).x, 2)
-        self.assertEqual(getitem(aList, 5).x, 2)
-
     def test_default_initialize_class_with_type_members(self):
         class C(Class, Final):
             t = Member(Value(float))
@@ -2249,3 +2240,194 @@ class TestClassCompilationCompilation(unittest.TestCase):
         # if we know it as Base1, but it's a Child, we should be able
         # to cross call it
         assert callFromBase1(callAsBase2, Child()) == 3
+
+    def test_class_nonempty(self):
+        class CNonempty(Class, Final):
+            x = Member(str, nonempty=True)
+
+            def __init__(self):
+                pass
+
+        class CEmpty(Class, Final):
+            x = Member(str)
+
+            def __init__(self):
+                pass
+
+        @Entrypoint
+        def callIt(X):
+            return X()
+
+        @Entrypoint
+        def deleteX(x):
+            del x.x
+
+        @Entrypoint
+        def getX(x):
+            return x.x
+
+        @Entrypoint
+        def setX(x, xVal):
+            x.x = xVal
+
+        cN = callIt(CNonempty)
+        cE = callIt(CEmpty)
+
+        assert getX(cN) == ""
+        assert getX(cE) == ""
+
+        with self.assertRaisesRegex(Exception, "Attribute 'x' cannot be deleted"):
+            deleteX(cN)
+        deleteX(cE)
+
+        with self.assertRaisesRegex(Exception, "Attribute 'x' is not initialized"):
+            deleteX(cE)
+
+        with self.assertRaisesRegex(Exception, "Attribute 'x' is not initialized"):
+            getX(cE)
+
+        setX(cE, "hihi")
+        assert getX(cE) == "hihi"
+
+    def test_class_nonempty_forces_construction(self):
+        class SomeOtherClass(Class):
+            pass
+
+        class CNonempty(Class, Final):
+            x = Member(SomeOtherClass, nonempty=True)
+
+            def __init__(self):
+                pass
+
+        class CEmpty(Class, Final):
+            x = Member(SomeOtherClass)
+
+            def __init__(self):
+                pass
+
+        @Entrypoint
+        def callIt(X):
+            return X()
+
+        callIt(CNonempty).x
+
+        with self.assertRaisesRegex(Exception, "Attribute 'x' is not initialized"):
+            callIt(CEmpty).x
+
+    def test_class_hierarchy(self):
+        class Base(Class):
+            def f(self) -> int:
+                return 0
+
+            def g(self) -> int:
+                return self.f()
+
+        class C1(Base):
+            def f(self) -> int:
+                return 1
+
+            def g(self) -> int:
+                return self.f()
+
+        def callG(x: Base):
+            return x.g()
+
+        assert callG(C1()) == Entrypoint(callG)(C1())
+
+    def test_convert_refTo(self):
+        class C(Class, Final):
+            x = Member(int)
+
+            def f(self):
+                return self.x
+
+        def f(c):
+            return c.f()
+
+        @Entrypoint
+        def doIt():
+            c = C(x=10)
+            return f(refTo(c))
+
+        assert doIt() == 10
+
+    def test_call_overridden_fun(self):
+        class C(Class):
+            def getIt(self) -> int:
+                return 0
+
+        class B(C, Final):
+            def getIt(self) -> int:
+                return 1
+
+        @Entrypoint
+        def f(c: C):
+            return c.getIt()
+
+        f(C())
+
+    def test_isinstance_perf(self):
+        class C(Class):
+            pass
+
+        class D(C, Final):
+            pass
+
+        class E(C, Final):
+            pass
+
+        @Entrypoint
+        def countIt(times, c: C):
+            isC = 0
+            isD = 0
+            isE = 0
+            isInt = 0
+
+            for _ in range(times):
+                if isinstance(c, C):
+                    isC += 1
+                if isinstance(c, D):
+                    isD += 1
+                if isinstance(c, E):
+                    isE += 1
+                if isinstance(c, int):
+                    isInt += 1
+
+            return (isC, isD, isE, isInt)
+
+        countIt(1, D())
+        t0 = time.time()
+        countIt(1000000, D())
+        print(time.time() - t0, " to do 1mm")
+
+        # I get 0.002 on my machine. This should be very fast
+        # because either the compiler can figure it out
+        assert time.time() - t0 < 0.01
+
+    def test_unresolved_forwards_in_class_type_signatures(self):
+        UnresolvedForward = Forward("UnresolvedForward")
+
+        class AClass(Class, Final):
+            def f(self, other: UnresolvedForward):
+                return "BAD"
+
+        @Entrypoint
+        def tryToCallF():
+            return AClass().f(10)
+
+        with self.assertRaisesRegex(TypeError, "unresolved forwards"):
+            tryToCallF()
+
+    def test_unresolved_forwards_in_function_type_signatures(self):
+        UnresolvedForward = Forward("UnresolvedForward")
+
+        @Function
+        def f(x: UnresolvedForward):
+            return "OK"
+
+        @Entrypoint
+        def tryToCallF():
+            f(10)
+
+        with self.assertRaisesRegex(TypeError, "has unresolved forwards"):
+            tryToCallF()
