@@ -11,6 +11,7 @@
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
+import typed_python
 
 from typed_python import (
     OneOf, TupleOf, Forward, ConstDict, Class, Final, Member,
@@ -22,6 +23,9 @@ import typed_python._types as _types
 import unittest
 
 
+typeWrapper = lambda t: typed_python.compiler.python_object_representation.typedPythonTypeToTypeWrapper(t)
+
+
 # a simple recursive 'value' type for testing
 Value = Forward("Value")
 
@@ -68,6 +72,13 @@ class ClassB(Class, Final):
 
 
 class TestOneOfCompilation(unittest.TestCase):
+    def test_oneof_wrapper_fast_is_check(self):
+        assert typeWrapper(OneOf(int, float))._simpleNoneCheckIndex() == -1
+        assert typeWrapper(OneOf(int, None))._simpleNoneCheckIndex() == 1
+        assert typeWrapper(OneOf(int, type(None)))._simpleNoneCheckIndex() == 1
+        assert typeWrapper(OneOf(None, int))._simpleNoneCheckIndex() == 0
+        assert typeWrapper(OneOf(None, object))._simpleNoneCheckIndex() is None
+
     def test_one_of_basic(self):
         @Compiled
         def f(x: OneOf(int, float)) -> OneOf(int, float):
@@ -661,3 +672,23 @@ class TestOneOfCompilation(unittest.TestCase):
 
         assert split("AHIB", "HI") == list
         assert split(b"AHIB", b"HI") == list
+
+    def test_isinstance_convert_overlapping(self):
+        @Entrypoint
+        def checkIt(x: OneOf(1, 2, 3, int)):
+            # this verifies that the compiler understands that just because something
+            # is an 'int' doesn't mean it's the 4th slot of the OneOf.
+            if isinstance(x, int):
+                return x
+
+            return 10
+
+        assert checkIt(2) == 2
+
+    def test_call_function_with_none_and_ifcheck(self):
+        @Entrypoint
+        def checkIt(x, y: object):
+            if x is None:
+                x = y
+
+        checkIt(None, float)
