@@ -13,12 +13,12 @@
 #   limitations under the License.
 
 from typed_python.compiler.global_variable_definition import GlobalVariableMetadata
+from typed_python.compiler.merge_type_wrappers import mergeTypeWrappers
 from typed_python.compiler.typed_expression import TypedExpression
 from typed_python.compiler.conversion_level import ConversionLevel
 from typed_python.compiler.type_wrappers.refcounted_wrapper import RefcountedWrapper
 from typed_python.compiler.type_wrappers.bound_method_wrapper import BoundMethodWrapper
 from typed_python.compiler.type_wrappers.python_typed_function_wrapper import PythonTypedFunctionWrapper
-from typed_python.compiler.type_wrappers.one_of_wrapper import OneOfWrapper
 import typed_python.compiler.type_wrappers.runtime_functions as runtime_functions
 from typed_python.compiler.type_wrappers.class_or_alternative_wrapper_mixin import (
     ClassOrAlternativeWrapperMixin
@@ -26,7 +26,7 @@ from typed_python.compiler.type_wrappers.class_or_alternative_wrapper_mixin impo
 from typed_python.compiler.type_wrappers.voidptr_masquerading_as_tp_type import (
     VoidPtrMasqueradingAsTPType
 )
-from typed_python import _types, PointerTo, Int32, Tuple, NamedTuple, bytecount
+from typed_python import _types, PointerTo, Int32, Tuple, NamedTuple, bytecount, RefTo
 
 import typed_python.compiler.native_ast as native_ast
 import typed_python.compiler
@@ -64,7 +64,8 @@ vtable_type = native_ast.Type.Struct(
         ('heldTypePtr', native_ast.VoidPtr),
         ('destructorFun', native_destructor_function_type),
         ('classDispatchTable', class_dispatch_table_type.pointer()),
-        ('initializationBitsBytecount', native_ast.Int64)
+        ('initializationBitsBytecount', native_ast.Int64),
+        ('classTypePtr', native_ast.VoidPtr)
     ],
     name="VTable"
 )
@@ -90,9 +91,15 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
         element_types = [('refcount', native_ast.Int64), ('vtable', vtable_type.pointer()), ('data', native_ast.UInt8)]
 
-        # this follows the general layout of 'held class' which is 1 bit per field for initialization and then
-        # each field packed directly according to byte size
-        self.bytesOfInitBits = (len(self.classType.MemberNames) + 7) // 8
+        # this follows the general layout of 'held class' which is 1 bit per field
+        # for initialization and then each field packed directly according to byte size
+        if all(
+            self.classType.ClassMembers[self.classType.MemberNames[i]].isNonempty
+            for i in range(len(self.classType.MemberNames))
+        ):
+            self.bytesOfInitBits = 0
+        else:
+            self.bytesOfInitBits = (len(self.classType.MemberNames) + 7) // 8
 
         byteOffset = 0
 
@@ -112,13 +119,41 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
             )
         ).load()
 
+    def fieldGuaranteedInitialized(self, ix):
+        if self.classType.MemberNames[ix] not in self.typeRepresentation.ClassMembers:
+            raise Exception(
+                f"Can't find "
+                f"{self.typeRepresentation.MemberNames[ix]} in "
+                f"{list(self.typeRepresentation.ClassMembers)}"
+            )
+        if self.typeRepresentation.ClassMembers[
+            self.classType.MemberNames[ix]
+        ].isNonempty:
+            return True
+
+        return False
+
+    def convert_refTo(self, context, instance):
+        refToType = RefTo(self.typeRepresentation.HeldClass)
+
+        return TypedExpression(
+            context,
+            (
+                self.get_layout_pointer(instance)
+                .ElementPtrIntegers(0, 2)
+                .cast(typeWrapper(refToType).getNativeLayoutType())
+            ),
+            refToType,
+            False
+        )
+
     def convert_pointerTo(self, context, instance):
         PtrT = PointerTo(self.typeRepresentation.HeldClass)
 
         return TypedExpression(
             context,
             (
-                self.get_layout_pointer(instance.nonref_expr)
+                self.get_layout_pointer(instance)
                 .ElementPtrIntegers(0, 2)
                 .cast(typeWrapper(PtrT).getNativeLayoutType())
             ),
@@ -145,6 +180,37 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
     def convert_fastnext(self, context, instance):
         return self.convert_method_call(context, instance, "__fastnext__", [], {})
 
+    def convert_to_type_as_expression(
+        self,
+        context,
+        expr,
+        target_type,
+        level: ConversionLevel,
+        mayThrowOnFailure=False,
+        assumeSuccessful=False
+    ):
+        if isinstance(target_type, ClassWrapper):
+            if target_type.typeRepresentation in self.typeRepresentation.MRO:
+                # this is an upcast
+                index = _types.getDispatchIndexForType(target_type.typeRepresentation, self.typeRepresentation)
+
+                return (
+                    TypedExpression(
+                        context,
+                        target_type.withDispatchIndex(expr, index),
+                        target_type,
+                        False
+                    ),
+                    context.constant(True)
+                )
+
+            if target_type.typeRepresentation.IsFinal:
+                if assumeSuccessful:
+                    return (
+                        target_type.stripClassDispatchIndex(context, expr),
+                        context.constant(True)
+                    )
+
     def convert_to_type_with_target(self, context, instance, targetVal, conversionLevel, mayThrowOnFailure=False):
         otherType = targetVal.expr_type
 
@@ -162,14 +228,45 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
                 return context.constant(True)
 
-            return context.pushPod(
-                bool,
-                runtime_functions.classObjectDowncast.call(
-                    instance.nonref_expr.cast(native_ast.VoidPtr),
-                    targetVal.expr.cast(native_ast.VoidPtr),
-                    context.getTypePointer(targetVal.expr_type.typeRepresentation)
+            if otherType.typeRepresentation.IsFinal:
+                initialized = context.allocateUninitializedSlot(bool)
+
+                with context.ifelse(
+                    self.get_class_type_ptr_as_voidptr(instance).cast(native_ast.UInt64).eq(
+                        context.getTypePointer(targetVal.expr_type.typeRepresentation)
+                        .cast(native_ast.UInt64)
+                    )
+                ) as (ifTrue, ifFalse):
+                    with ifTrue:
+                        context.pushEffect(
+                            targetVal.expr.store(
+                                self.get_layout_pointer(instance)
+                            )
+                        )
+                        context.pushEffect(
+                            initialized.expr.store(
+                                native_ast.const_bool_expr(True)
+                            )
+                        )
+                        targetVal.convert_incref()
+
+                    with ifFalse:
+                        context.pushEffect(
+                            initialized.expr.store(
+                                native_ast.const_bool_expr(False)
+                            )
+                        )
+
+                return initialized
+            else:
+                return context.pushPod(
+                    bool,
+                    runtime_functions.classObjectDowncast.call(
+                        instance.nonref_expr.cast(native_ast.VoidPtr),
+                        targetVal.expr.cast(native_ast.VoidPtr),
+                        context.getTypePointer(targetVal.expr_type.typeRepresentation)
+                    )
                 )
-            )
 
         return super().convert_to_type_with_target(context, instance, targetVal, conversionLevel, mayThrowOnFailure)
 
@@ -189,13 +286,25 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         This function will return a TypedExpression, or None if it set an exception."""
         return self.convert_method_call(context, instance, "__typed_python_int_iter_value__", [], {})
 
-    def get_layout_pointer(self, nonref_expr):
+    def get_layout_pointer(self, instance):
+        # diagnostic you can use to check whether our dispatch indices are getting
+        # messed up.
+        # if self.typeRepresentation.IsFinal:
+        #     from typed_python import UInt64
+        #     c = instance.context
+        #     with c.ifelse(self.get_dispatch_index(instance)) as (ifTrue, ifFalse):
+        #         with ifTrue:
+        #             c.logDiagnostic(str(c.functionContext), ": ", str(self), ":",
+        #                 c.pushPod(UInt64, self.get_dispatch_index(instance))
+        #             )
+
+        return self.get_layout_pointer_native(instance.nonref_expr)
+
+    def get_layout_pointer_native(self, nonref_expr):
         # our layout is 48 bits of pointer and 16 bits of classDispatchTableIndex.
         # so whenever we interact with the pointer we need to chop off the top 16 bits
-
-        # TODO: seems like this should be legal?
-        # if self.typeRepresentation.IsFinal:
-        #    return nonref_expr.cast(self.layoutType)
+        if self.typeRepresentation.IsFinal:
+            return nonref_expr.cast(self.layoutType)
 
         return (
             nonref_expr
@@ -206,7 +315,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
     def classDispatchTable(self, instance):
         classDispatchTables = (
-            self.get_layout_pointer(instance.nonref_expr)
+            self.get_layout_pointer(instance)
             .ElementPtrIntegers(0, 1).load().ElementPtrIntegers(0, 2).load()
         )
 
@@ -248,7 +357,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
             return native_ast.const_uint64_expr(self.bytesOfInitBits)
         else:
             return (
-                self.get_layout_pointer(instance.nonref_expr)
+                self.get_layout_pointer(instance)
                 # get a pointer to the vtable
                 .ElementPtrIntegers(0, 1)
                 # load it
@@ -258,6 +367,17 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
                 .load()
             )
 
+    def get_class_type_ptr_as_voidptr(self, instance):
+        return (
+            self.get_layout_pointer(instance)
+            # get a pointer to the vtable
+            .ElementPtrIntegers(0, 1)
+            # load it
+            .load()
+            .ElementPtrIntegers(0, 4)
+            .load()
+        )
+
     def get_refcount_ptr_expr(self, nonref_expr):
         """Return a pointer to the object's refcount. Subclasses can override.
 
@@ -265,7 +385,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
             nonref_expr - a native expression equivalent to 'self.nonref_expr'. In most cases
                 this will be the pointer to the actual refcounted data structure.
         """
-        return self.get_layout_pointer(nonref_expr).ElementPtrIntegers(0, 0)
+        return self.get_layout_pointer_native(nonref_expr).ElementPtrIntegers(0, 0)
 
     def get_dispatch_index(self, instance):
         """Return the integer index of the current class dispatch within this instances' vtable."""
@@ -289,28 +409,33 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         return self.layoutType
 
     def on_refcount_zero(self, context, instance):
-        vtablePtr = (
-            self.get_layout_pointer(instance.nonref_expr)
-            .ElementPtrIntegers(0, 1)
-            .load()
-        )
+        if self.typeRepresentation.IsFinal:
+            self.generateNativeDestructorFunction(context, None, instance)
+        else:
+            vtablePtr = (
+                self.get_layout_pointer(instance)
+                .ElementPtrIntegers(0, 1)
+                .load()
+            )
 
-        destructorPtr = (
-            vtablePtr
-            .ElementPtrIntegers(0, 1)
-            .load()
-        )
+            destructorPtr = (
+                vtablePtr
+                .ElementPtrIntegers(0, 1)
+                .load()
+            )
 
-        with context.ifelse(destructorPtr.cast(native_ast.Int64)) as (ifTrue, ifFalse):
-            with ifFalse:
-                # we have an empty slot. We need to compile it
-                context.pushEffect(
-                    runtime_functions.compileClassDestructor.call(
-                        vtablePtr.cast(native_ast.VoidPtr)
+            with context.ifelse(destructorPtr.cast(native_ast.Int64)) as (ifTrue, ifFalse):
+                with ifFalse:
+                    # we have an empty slot. We need to compile it
+                    context.pushEffect(
+                        runtime_functions.compileClassDestructor.call(
+                            vtablePtr.cast(native_ast.VoidPtr)
+                        )
                     )
-                )
 
-        return native_ast.CallTarget.Pointer(destructorPtr).call(instance.expr.cast(native_ast.VoidPtr))
+            return native_ast.CallTarget.Pointer(destructorPtr).call(
+                instance.expr.cast(native_ast.VoidPtr)
+            )
 
     def compileDestructor(self, converter):
         return converter.defineNativeFunction(
@@ -322,6 +447,8 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         )
 
     def generateNativeDestructorFunction(self, context, out, instance):
+        instance = self.stripClassDispatchIndex(context, instance)
+
         for i in range(len(self.typeRepresentation.MemberTypes)):
             if not typeWrapper(self.typeRepresentation.MemberTypes[i]).is_pod:
                 with context.ifelse(context.pushPod(bool, self.isInitializedNativeExpr(instance, i))) as (true_block, false_block):
@@ -330,15 +457,14 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
                             self.convert_attribute(context, instance, i, nocheck=True).convert_destroy()
                         )
 
-        context.pushEffect(runtime_functions.free.call(self.get_layout_pointer(instance.nonref_expr).cast(native_ast.UInt8Ptr)))
+        context.pushEffect(runtime_functions.free.call(self.get_layout_pointer(instance).cast(native_ast.UInt8Ptr)))
 
     def memberPtr(self, instance, ix):
         return (
-            self.get_layout_pointer(instance.nonref_expr)
+            self.get_layout_pointer(instance)
             .cast(native_ast.UInt8.pointer())
             .elemPtr(self.bytesOfInitBitsForInstance(instance))
-            .ElementPtrIntegers(self.indexToByteOffset[ix])
-            .ElementPtrIntegers(self.BYTES_BEFORE_INIT_BITS)
+            .ElementPtrIntegers(self.indexToByteOffset[ix] + self.BYTES_BEFORE_INIT_BITS)
             .cast(
                 typeWrapper(self.typeRepresentation.MemberTypes[ix])
                 .getNativeLayoutType()
@@ -347,30 +473,53 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         )
 
     def isInitializedNativeExpr(self, instance, ix):
+        if self.fieldGuaranteedInitialized(ix):
+            return native_ast.const_bool_expr(True)
+
         byte = ix // 8
         bit = ix % 8
 
         return (
-            self.get_layout_pointer(instance.nonref_expr)
+            self.get_layout_pointer(instance)
             .cast(native_ast.UInt8.pointer())
             .ElementPtrIntegers(self.BYTES_BEFORE_INIT_BITS + byte)
             .load()
             .rshift(native_ast.const_uint8_expr(bit))
             .bitand(native_ast.const_uint8_expr(1))
-        )
+        ).cast(native_ast.Bool)
 
     def setIsInitializedExpr(self, instance, ix):
+        if self.fieldGuaranteedInitialized(ix):
+            return native_ast.nullExpr
+
         byte = ix // 8
         bit = ix % 8
 
         bytePtr = (
-            self.get_layout_pointer(instance.nonref_expr)
+            self.get_layout_pointer(instance)
             .cast(native_ast.UInt8.pointer())
             .ElementPtrIntegers(self.BYTES_BEFORE_INIT_BITS + byte)
         )
 
         return bytePtr.store(bytePtr.load().bitor(native_ast.const_uint8_expr(1 << bit)))
 
+    def clearIsInitializedExpr(self, instance, ix):
+        if self.fieldGuaranteedInitialized(ix):
+            return native_ast.nullExpr
+
+        assert instance.isReference
+
+        byte = ix // 8
+        bit = ix % 8
+
+        bytePtr = (
+            self.get_layout_pointer(instance)
+            .cast(native_ast.UInt8.pointer())
+            .ElementPtrIntegers(self.BYTES_BEFORE_INIT_BITS + byte)
+        )
+
+        return bytePtr.store(bytePtr.load().bitand(native_ast.const_uint8_expr(255 - (1 << bit))))
+
     def convert_attribute(self, context, instance, attribute, nocheck=False):
         if attribute in self.typeRepresentation.MemberFunctions:
             methodType = BoundMethodWrapper(_types.BoundMethod(self.typeRepresentation, attribute))
@@ -398,7 +547,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         return context.pushReference(
             self.typeRepresentation.MemberTypes[ix],
             self.memberPtr(instance, ix)
-        ).heldToRef()
+        )
 
     def resultTypesForCall(self, func, argTypes, kwargTypes):
         resultTypes = set()
@@ -491,7 +640,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
         # compute the return type of dispatching to this function. it will be a one-of if we
         # might dispatch to multiple possible functions.
-        output_type = OneOfWrapper.mergeTypes(resultTypes)
+        output_type = mergeTypeWrappers(resultTypes)
 
         argSignatureStrings = [str(x) for x in argTypes[1:]]
         argSignatureStrings.extend([f"{k}={v}" for k, v in kwargTypes.items()])
@@ -736,6 +885,30 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
         return res
 
+    def stripClassDispatchIndex(self, context, instance):
+        """Return 'instance' with a class-dispatch of 0.
+
+        When we call a method that's overridden in a child class, we need to ensure
+        that the pointer argument matches the type that the child is expecting.
+
+        For instance, if class B is the base, C is the child, and they have
+        'f(self)' defined, both B and C's version of 'f' expect to receive an instance
+        with their own type. However, if we have an instance of C which we know as
+        B, then when we call 'f', C's version of 'f' will get 'self' masquerading as
+        C with a pointer dispatch of 1.
+
+        This function strips that 1 off.
+        """
+        return TypedExpression(
+            context,
+            instance.nonref_expr
+            .cast(native_ast.UInt64)
+            .bitand(native_ast.const_uint64_expr(0xFFFFFFFFFFFF))  # 48 bits of 1s
+            .cast(self.layoutType),
+            self,
+            False
+        )
+
     @staticmethod
     def compileMethodInstantiation(
         converter, interfaceClass, implementingClass,
@@ -772,12 +945,11 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
         assert bytecount(pyImpl.ClosureType) == 0, "Class methods should have empty closures."
 
-        return typeWrapper(pyImpl).compileCall(converter, retType, argTypes, kwargTypes, False)
+        return typeWrapper(pyImpl).compileCall(
+            converter, retType, argTypes, kwargTypes, False, stripFirstArgClassDispatchIndex=True
+        )
 
     def convert_set_attribute(self, context, instance, attribute, value):
-        if value is not None:
-            value = value.refToHeld()
-
         if not isinstance(attribute, int):
             ix = self.nameToIndex.get(attribute)
         else:
@@ -797,15 +969,44 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
 
         attr_type = typeWrapper(self.typeRepresentation.MemberTypes[ix])
 
+        if value is None:
+            # we're deleting this attribute
+            if self.fieldGuaranteedInitialized(ix):
+                return context.pushException(
+                    AttributeError,
+                    f"Attribute '{self.classType.MemberNames[ix]}' cannot be deleted"
+                )
+
+            with context.ifelse(context.pushPod(bool, self.isInitializedNativeExpr(instance, ix))) as (
+                true_block, false_block
+            ):
+                with true_block:
+                    if not attr_type.is_pod:
+                        member = context.pushReference(attr_type, self.memberPtr(instance, ix))
+                        member.convert_destroy()
+
+                    context.pushEffect(
+                        self.clearIsInitializedExpr(instance, ix)
+                    )
+
+                with false_block:
+                    context.pushException(
+                        AttributeError,
+                        f"Attribute '{self.classType.MemberNames[ix]}' is not initialized"
+                    )
+
+            return context.constant(None)
+
         value = value.convert_to_type(attr_type, ConversionLevel.ImplicitContainers)
         if value is None:
             return None
 
         if attr_type.is_pod:
-            return context.pushEffect(
+            context.pushEffect(
                 self.memberPtr(instance, ix).store(value.nonref_expr)
                 >> self.setIsInitializedExpr(instance, ix)
             )
+            return context.constant(None)
         else:
             member = context.pushReference(attr_type, self.memberPtr(instance, ix))
 
@@ -818,7 +1019,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
                         self.setIsInitializedExpr(instance, ix)
                     )
 
-            return native_ast.nullExpr
+            return context.constant(None)
 
     def convert_type_call(self, context, typeInst, args, kwargs):
         # pack the named arguments onto the back of the call, and pass
@@ -875,7 +1076,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
             )
 
         for i in range(len(self.classType.MemberTypes)):
-            if _types.wantsToDefaultConstruct(self.classType.MemberTypes[i]):
+            if _types.wantsToDefaultConstruct(self.classType.MemberTypes[i]) or self.fieldGuaranteedInitialized(i):
                 name = self.classType.MemberNames[i]
 
                 if name in self.classType.MemberDefaultValues:
@@ -931,7 +1132,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         if self.has_method("__hash__"):
             return self.convert_method_call(context, expr, "__hash__", (), {})
 
-        layoutPtr = self.get_layout_pointer(expr.nonref_expr).cast(native_ast.UInt64)
+        layoutPtr = self.get_layout_pointer(expr).cast(native_ast.UInt64)
 
         return context.pushPod(
             Int32,
@@ -945,15 +1146,7 @@ class ClassWrapper(ClassOrAlternativeWrapperMixin, RefcountedWrapper):
         if self.typeRepresentation.IsFinal:
             return context.constant(self.typeRepresentation)
 
-        vtablePtr = (
-            self.get_layout_pointer(instance.nonref_expr)
-            .ElementPtrIntegers(0, 1)
-            .load()
-        )
-
         return context.pushPod(
             VoidPtrMasqueradingAsTPType(self.typeRepresentation),
-            runtime_functions.classTypeAsPointer.call(
-                vtablePtr.cast(native_ast.VoidPtr)
-            )
+            self.get_class_type_ptr_as_voidptr(instance)
         )
