@@ -18,7 +18,7 @@ from typed_python.compiler.conversion_level import ConversionLevel
 import typed_python.compiler.type_wrappers.runtime_functions as runtime_functions
 from typed_python.compiler.type_wrappers.bound_method_wrapper import BoundMethodWrapper
 from typed_python.compiler.type_wrappers.wrapper import Wrapper
-from typed_python.compiler.type_wrappers.native_hash import table_next_slot, table_clear, table_contains, \
+from typed_python.compiler.type_wrappers.hash_table_implementation import table_next_slot, table_clear, table_contains, \
     dict_delitem, dict_getitem, dict_get, dict_setitem
 from typed_python import Tuple, PointerTo, Int32, UInt8, Dict, ConstDict
 
