@@ -13,7 +13,7 @@
 #   limitations under the License.
 
 from typed_python.compiler.type_wrappers.wrapper import Wrapper
-
+from typed_python.compiler.merge_type_wrappers import mergeTypeWrappers
 from typed_python import _types, OneOf, PointerTo
 from typed_python.compiler.typed_expression import TypedExpression
 from typed_python.compiler.conversion_level import ConversionLevel
@@ -56,39 +56,6 @@ class OneOfWrapper(Wrapper):
     def getNativeLayoutType(self):
         return self.layoutType
 
-    @staticmethod
-    def mergeTypes(types):
-        """Produce a canonical 'OneOf' type wrapper from all of the arguments.
-
-        If there is only one argument, it will not be a one-of. If there are no arguments,
-        we return None.
-        """
-        if not types:
-            return None
-
-        if all(isinstance(t, Wrapper) for t in types):
-            types = list(set(types))
-            if len(types) == 1:
-                return types[0]
-
-        allTypes = set()
-        for t in types:
-            if isinstance(t, Wrapper):
-                t = t.interpreterTypeRepresentation
-
-            if isinstance(t, OneOf):
-                allTypes.update(t.Types)
-            else:
-                allTypes.add(t)
-
-        if not allTypes:
-            return None
-
-        if len(allTypes) == 1:
-            return typeWrapper(list(allTypes)[0])
-
-        return typeWrapper(OneOf(*sorted(allTypes, key=str)))
-
     def convert_which_native(self, expr):
         return expr.ElementPtrIntegers(0, 0).load()
 
@@ -114,7 +81,7 @@ class OneOfWrapper(Wrapper):
                         assert isinstance(t, Wrapper)
                         types.append(t)
 
-            output_type = OneOfWrapper.mergeTypes(types)
+            output_type = mergeTypeWrappers(types)
 
             if output_type is None:
                 return None
@@ -239,9 +206,61 @@ class OneOfWrapper(Wrapper):
             ("oneof", self, "unaryop", op)
         )
 
+    def _simpleNoneCheckIndex(self):
+        """Returns the index of the oneof that matches None.
+
+        If no index matches None, returns -1.
+
+        If one of the types could match None (because it's object),
+        returns None.
+        """
+        noneIx = -1
+
+        for ix in range(len(self.typeRepresentation.Types)):
+            T = self.typeRepresentation.Types[ix]
+
+            if T is type(None):  # noqa
+                if noneIx is None:
+                    raise Exception(
+                        f"{self} has two slots that could be None, which makes no sense"
+                    )
+
+                noneIx = ix
+            elif isinstance(T, type) and issubclass(type(None), T):
+                return None
+
+        return noneIx
+
     def convert_bin_op(self, context, left, op, right, inplace):
+        if (
+            (op.matches.Is or op.matches.IsNot)
+            and right.expr_type.typeRepresentation is type(None)  # noqa
+        ):
+            # if one of our fields is actually None, and none of them is object,
+            # then we can answer this by just checking our slot type
+            index = self._simpleNoneCheckIndex()
+            if index is not None:
+                if index == -1:
+                    # none of our fields can match 'None'
+                    return context.constant(op.matches.IsNot)
+
+                if op.matches.Is:
+                    return context.pushPod(
+                        bool,
+                        self.convert_which_native(left.expr).eq(
+                            native_ast.const_uint8_expr(index)
+                        )
+                    )
+                else:
+                    return context.pushPod(
+                        bool,
+                        self.convert_which_native(left.expr).neq(
+                            native_ast.const_uint8_expr(index)
+                        )
+                    )
+
         return context.expressionAsFunctionCall(
-            "oneof_binop",
+            "oneof_binop_" + type(op).__name__,
             (left, right),
             lambda left, right: self.unwrap(
                 left.context,
@@ -287,13 +306,26 @@ class OneOfWrapper(Wrapper):
             expr.convert_copy_initialize(other)
             temp.convert_destroy()
 
-    def refAsType(self, context, expr, subtype):
-        """Unwrap this assuming that it is _definitely_ 'subtype' which must be a TP type."""
-        for ix in range(len(self.typeRepresentation.Types)):
-            if self.typeRepresentation.Types[ix] == subtype:
-                return self.refAs(context, expr, ix)
+    def unambiguousTypeIndexFor(self, subtype):
+        """Given a TP type 'subtype', is there one and only one of our types it could be?
 
-        raise Exception(f"type {subtype} is not one of the options in {self}")
+        Returns None if we can't pick one, or the index if we can.
+        """
+        isDefinitely = []
+        for index, eltType in enumerate(self.typeRepresentation.Types):
+            canConvert = typeWrapper(eltType).can_convert_to_type(
+                typeWrapper(subtype), ConversionLevel.Signature
+            )
+
+            if canConvert is True:
+                isDefinitely.append(index)
+            elif canConvert == "Maybe":
+                return None
+
+        if len(isDefinitely) == 1:
+            return isDefinitely[0]
+
+        return None
 
     def refAs(self, context, expr, which):
         assert expr.expr_type == self
@@ -307,25 +339,53 @@ class OneOfWrapper(Wrapper):
         )
 
     def convert_copy_initialize(self, context, expr, other):
-        assert expr.isReference
-        assert other.expr_type == self
-
         if self.is_pod:
             context.pushEffect(
                 expr.expr.store(other.nonref_expr)
             )
-        else:
-            with context.switch(other.expr.ElementPtrIntegers(0, 0).load(),
-                                range(len(self.typeRepresentation.Types)),
-                                False) as indicesAndContexts:
-                for ix, subcontext in indicesAndContexts:
-                    with subcontext:
-                        self.refAs(context, expr, ix).convert_copy_initialize(self.refAs(context, other, ix))
-                        context.pushEffect(
-                            expr.expr.ElementPtrIntegers(0, 0).store(native_ast.const_uint8_expr(ix))
-                        )
+            return
 
-    def convert_destroy(self, context, expr):
+        res = context.expressionAsFunctionCall(
+            "copy_initialize_" + str(self),
+            (expr, other),
+            lambda instance, other: self.convert_copy_initialize_inner(instance.context, instance, other),
+            ("copy_initialize_", self),
+            outputType=context.constant(None).expr_type
+        )
+
+        if res is not None:
+            context.pushEffect(res.expr)
+
+    def convert_copy_initialize_inner(self, context, expr, other):
+        assert expr.isReference
+        assert other.expr_type == self
+
+        with context.switch(other.expr.ElementPtrIntegers(0, 0).load(),
+                            range(len(self.typeRepresentation.Types)),
+                            False) as indicesAndContexts:
+            for ix, subcontext in indicesAndContexts:
+                with subcontext:
+                    self.refAs(context, expr, ix).convert_copy_initialize(self.refAs(context, other, ix))
+                    context.pushEffect(
+                        expr.expr.ElementPtrIntegers(0, 0).store(native_ast.const_uint8_expr(ix))
+                    )
+
+    def convert_destroy(self, context, target):
+        if self.is_pod:
+            return
+
+        res = context.expressionAsFunctionCall(
+            "decref_" + str(self),
+            (target,),
+            lambda instance: self.convert_destroy_inner(instance.context, instance),
+            ("decref", self),
+            outputType=context.constant(None).expr_type
+        )
+
+        if res is not None:
+            context.pushEffect(res.expr)
+
+    def convert_destroy_inner(self, context, expr):
         if not self.is_pod:
             with context.switch(expr.expr.ElementPtrIntegers(0, 0).load(),
                                 range(len(self.typeRepresentation.Types)),
@@ -349,6 +409,13 @@ class OneOfWrapper(Wrapper):
 
         return "Maybe"
 
+    def convert_to_type_as_expression(self, context, expr, targetType, level, mayThrowOnFailure, assumeSuccessful):
+        if assumeSuccessful and level == ConversionLevel.Signature:
+            ix = self.unambiguousTypeIndexFor(targetType)
+
+            if ix is not None:
+                return self.refAs(context, expr, ix), context.constant(True)
+
     def convert_to_type_with_target(self, context, expr, targetVal, conversionLevel, mayThrowOnFailure=False):
         isInitialized = context.push(bool, lambda tgt: tgt.expr.store(native_ast.const_bool_expr(False)))
 
