@@ -14,11 +14,11 @@
 
 
 from typed_python import PointerTo, bytecount, NamedTuple, Class
+from typed_python.compiler.merge_type_wrappers import mergeTypeWrappers
 from typed_python._types import is_default_constructible
 from typed_python.internals import CellAccess
 from typed_python.compiler.conversion_level import ConversionLevel
 from typed_python.compiler.type_wrappers.wrapper import Wrapper
-from typed_python.compiler.type_wrappers.one_of_wrapper import OneOfWrapper
 from typed_python.compiler.type_wrappers.typed_tuple_masquerading_as_tuple_wrapper import TypedTupleMasqueradingAsTuple
 from typed_python.compiler.type_wrappers.named_tuple_masquerading_as_dict_wrapper import NamedTupleMasqueradingAsDict
 import typed_python.compiler.type_wrappers.runtime_functions as runtime_functions
@@ -90,7 +90,7 @@ class PythonTypedFunctionWrapper(Wrapper):
                 else:
                     expr = expr.expr_type.refHeld(expr)
             elif isinstance(pathElt, type):
-                expr.changeType(typeWrapper(pathElt))
+                expr = expr.changeType(typeWrapper(pathElt))
             else:
                 assert False, "invalid cell path"
 
@@ -397,7 +397,15 @@ class PythonTypedFunctionWrapper(Wrapper):
                 return typeWrapper(overloadArg.typeFilter)
             return argType
 
-    def compileCall(self, converter, returnType, argTypes, kwargTypes, provideClosureArgument):
+    def compileCall(
+        self,
+        converter,
+        returnType,
+        argTypes,
+        kwargTypes,
+        provideClosureArgument,
+        stripFirstArgClassDispatchIndex=False
+    ):
         """Compile this function being called with a particular signature.
 
         Args:
@@ -409,6 +417,10 @@ class PythonTypedFunctionWrapper(Wrapper):
                 arguments we're passing.
             provideClosureArgument - if True, then the first argument is of our closure type.
                 If false, then our closure type must be empty, and no argument for it is provided.
+            stripFirstArgClassDispatchIndex - if True, then the first argument is a class 'self', and
+                we want to strip out its class dispatch index, because it was dispatched
+                from code that knew that class as one of 'self''s base-classes. In fact,
+                the dispatch index should be zero.
         Returns:
             a TypedCallTarget, or None
         """
@@ -421,7 +433,7 @@ class PythonTypedFunctionWrapper(Wrapper):
                 kwargTypes
             )
 
-            returnType = OneOfWrapper.mergeTypes(possibleTypes)
+            returnType = mergeTypeWrappers(possibleTypes)
 
             if returnType is None:
                 return None
@@ -436,7 +448,14 @@ class PythonTypedFunctionWrapper(Wrapper):
             ([self] if provideClosureArgument else []) + list(argTypes) + list(kwargTypes.values()),
             returnType,
             lambda context, outputVar, *args: (
-                self.generateMethodImplementation(context, returnType, args, argNames, provideClosureArgument)
+                self.generateMethodImplementation(
+                    context,
+                    returnType,
+                    args,
+                    argNames,
+                    provideClosureArgument,
+                    stripFirstArgClassDispatchIndex=stripFirstArgClassDispatchIndex
+                )
             )
         )
 
@@ -556,7 +575,10 @@ class PythonTypedFunctionWrapper(Wrapper):
 
         return None
 
-    def generateMethodImplementation(self, context, returnType, args, argNames, provideClosureArgument):
+    def generateMethodImplementation(
+        self, context, returnType, args, argNames,
+        provideClosureArgument, stripFirstArgClassDispatchIndex=False
+    ):
         """Generate native code that calls us with a given return type and set of arguments.
 
         We try each overload with successively stronger type conversions. The first one that
@@ -573,6 +595,10 @@ class PythonTypedFunctionWrapper(Wrapper):
                 argument, or the name of the argument if passed as a keyword argument.
             provideClosureArgument - if True, then the first argument is of our closure type.
                 If false, then our closure type must be empty, and no argument for it is provided.
+            stripFirstArgClassDispatchIndex - if True, then the first argument is a class 'self', and
+                we want to strip out its class dispatch index, because it was dispatched
+                from code that knew that class as one of 'self''s base-classes. In fact,
+                the dispatch index should be zero.
         """
         func = self.typeRepresentation
 
@@ -582,6 +608,12 @@ class PythonTypedFunctionWrapper(Wrapper):
         else:
             closureArg = None
 
+        if stripFirstArgClassDispatchIndex:
+            args = (args[0].expr_type.stripClassDispatchIndex(
+                args[0].context,
+                args[0]
+            ),) + args[1:]
+
         argTypes = [a.expr_type for i, a in enumerate(args) if argNames[i] is None]
         kwargTypes = {argNames[i]: a.expr_type for i, a in enumerate(args) if argNames[i] is not None}
 
@@ -635,11 +667,12 @@ class PythonTypedFunctionWrapper(Wrapper):
 
                     # if we definitely match, we can return early
                     if mightMatch is True:
-                        context.pushException(
-                            TypeError,
-                            f"Thought we found overload for {self} matching {argTypes} and "
-                            f"{kwargTypes}, but it didn't accept the arguments."
-                        )
+                        if successful.constantValue is not True:
+                            context.pushException(
+                                TypeError,
+                                f"Thought we found overload for {self} matching {argTypes} and "
+                                f"{kwargTypes}, but it didn't accept the arguments."
+                            )
                         return
 
         # generate a cleanup handler for the cases where we don't match a method signature.
@@ -680,6 +713,8 @@ class PythonTypedFunctionWrapper(Wrapper):
         convertedArgs = []
         convertedKwargs = {}
 
+        argConversionMightNotBeSuccessful = False
+
         for argIx, argExpr in enumerate(args):
             if argNames[argIx] is None:
                 argType = argAndKwargTypes[0][argIx]
@@ -688,18 +723,34 @@ class PythonTypedFunctionWrapper(Wrapper):
 
             argType = typeWrapper(argType)
 
-            convertedArg = context.allocateUninitializedSlot(argType)
+            if argExpr.expr_type == argType:
+                # nothing to do
+                convertedArg = argExpr
+            else:
+                convertedArg = context.allocateUninitializedSlot(argType)
 
-            successful = argExpr.convert_to_type_with_target(convertedArg, conversionLevel)
+                successful = argExpr.convert_to_type_with_target(convertedArg, conversionLevel)
 
-            with context.ifelse(successful.nonref_expr) as (ifTrue, ifFalse):
-                with ifFalse:
-                    context.pushTerminal(
-                        native_ast.Expression.Return(arg=native_ast.const_bool_expr(False))
-                    )
+                if successful.isConstant:
+                    if successful.constantValue:
+                        context.markUninitializedSlotInitialized(convertedArg)
+                    else:
+                        # we can return early
+                        context.pushTerminal(
+                            native_ast.Expression.Return(arg=native_ast.const_bool_expr(False))
+                        )
+                        return
+                else:
+                    argConversionMightNotBeSuccessful = True
 
-                with ifTrue:
-                    context.markUninitializedSlotInitialized(convertedArg)
+                    with context.ifelse(successful.nonref_expr) as (ifTrue, ifFalse):
+                        with ifFalse:
+                            context.pushTerminal(
+                                native_ast.Expression.Return(arg=native_ast.const_bool_expr(False))
+                            )
+
+                        with ifTrue:
+                            context.markUninitializedSlotInitialized(convertedArg)
 
             if argNames[argIx] is None:
                 convertedArgs.append(convertedArg)
@@ -724,3 +775,8 @@ class PythonTypedFunctionWrapper(Wrapper):
         outputVar.changeType(typeWrapper(retType), True).convert_copy_initialize(res)
 
         context.pushReturnValue(context.constant(True))
+
+        if not argConversionMightNotBeSuccessful:
+            context.functionContext.functionMetadata.setConstantReturnValue(
+                True
+            )
