@@ -18,7 +18,7 @@ import typed_python.compiler.type_wrappers.runtime_functions as runtime_function
 from typed_python.compiler.type_wrappers.bound_method_wrapper import BoundMethodWrapper
 from typed_python.compiler.type_wrappers.wrapper import Wrapper
 from typed_python.compiler.conversion_level import ConversionLevel
-from typed_python.compiler.type_wrappers.native_hash import table_next_slot, table_clear, \
+from typed_python.compiler.type_wrappers.hash_table_implementation import table_next_slot, table_clear, \
     table_contains, set_add, set_add_or_remove, set_remove, set_discard, set_pop
 from typed_python import PointerTo, Int32, UInt8, ListOf, TupleOf, Set, Tuple, NamedTuple, Dict, ConstDict
 
