@@ -12,7 +12,7 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-from typed_python import UInt64, PointerTo, RefTo
+from typed_python import UInt64, PointerTo
 from typed_python.python_ast import BinaryOp, ComparisonOp, BooleanOp
 import typed_python.compiler.native_ast as native_ast
 import typed_python.compiler
@@ -61,6 +61,15 @@ class TypedExpression:
     def changeContext(self, newContext):
         return TypedExpression(newContext, self.expr, self.expr_type, self.isReference)
 
+    def withConstantValue(self, newVal):
+        return TypedExpression(
+            self.context,
+            self.expr,
+            self.expr_type,
+            self.isReference,
+            constantValue=newVal
+        )
+
     def asPointer(self):
         """Change from being a reference to 'T' to being a _value_ of type 'T*'"""
         assert self.isReference
@@ -114,24 +123,6 @@ class TypedExpression:
             True
         )
 
-    def refToHeld(self):
-        if getattr(self.expr_type.typeRepresentation, "__typed_python_category__", None) == "RefTo":
-            return TypedExpression(
-                self.context,
-                self.nonref_expr,
-                self.expr_type.typeRepresentation.ElementType,
-                True
-            )
-
-        return self
-
-    def heldToRef(self):
-        if getattr(self.expr_type.typeRepresentation, "__typed_python_category__", None) == "HeldClass":
-            assert self.isReference
-            return self.changeType(RefTo(self.expr_type.typeRepresentation), isReferenceOverride=False)
-
-        return self
-
     def changeType(self, newType, isReferenceOverride=None):
         """Return a TypedExpression with the same native_ast content but a different type-wrapper."""
         return TypedExpression(
@@ -279,7 +270,12 @@ class TypedExpression:
     def convert_method_call(self, methodname, args, kwargs):
         return self.expr_type.convert_method_call(self.context, self, methodname, args, kwargs)
 
-    def convert_to_type(self, target_type, conversionLevel: ConversionLevel):
+    def convert_to_type(
+        self,
+        target_type,
+        conversionLevel: ConversionLevel,
+        assumeSuccessful=False
+    ):
         """Convert to a target type as a function argument.
 
         If 'explicit', then allow conversions that may change type (e.g. int->float). Otherwise
@@ -287,7 +283,13 @@ class TypedExpression:
         """
         target_type = typeWrapper(target_type)
 
-        return self.expr_type.convert_to_type(self.context, self, target_type, conversionLevel)
+        return self.expr_type.convert_to_type(
+            self.context,
+            self,
+            target_type,
+            conversionLevel,
+            assumeSuccessful=assumeSuccessful
+        )
 
     def convert_context_manager_enter(self):
         return self.expr_type.convert_context_manager_enter(self.context, self)
