@@ -1,5 +1,7 @@
 """A dummy module for testing purposes."""
 import pytz
+import lz4
+import numpy
 
 
 def testfunction():
