@@ -34,7 +34,11 @@ class hash_table_layout {
         , hash_table_count(0)
         , hash_table_empty_slots(0) {}
 
-    enum { EMPTY = -1, DELETED = -2 };
+    // We follow the same hashing scheme as the internals of pythons dictionaries
+    // as detailed here:
+    //      https://hg.python.org/cpython/file/52f68c95e025/Objects/dictobject.c#l296
+
+    enum { EMPTY = -1, DELETED = -2, PERTURB_SHIFT = 5, MIN_SIZE = 8 };
 
     void setTo(int32_t* ptr, int32_t value, size_t count) {
         for (size_t k = 0; k < count; k++) {
@@ -49,41 +53,32 @@ class hash_table_layout {
             return -1;
         }
 
-        // because typed_python has to use python's mod, which is
-        // different than c++'s mod for negative numbers, we just map
-        // negatives to positives.
         if (hash < 0) {
             hash = -hash;
         }
 
-        int32_t offset = hash % hash_table_size;
+        uint64_t mask = hash_table_size - 1;
+        uint64_t perturb = hash;
+        uint64_t offset = hash;
 
         while (true) {
             // slot is empty
-            int32_t slot = hash_table_slots[offset];
+            int32_t slot = hash_table_slots[offset & mask];
 
             if (slot == EMPTY) {
                 return -1;
             }
 
-            if (slot != DELETED && hash_table_hashes[offset] == hash
+            if (slot != DELETED && hash_table_hashes[offset & mask] == hash
                 && compare(items + item_size * slot)) {
                 return slot;
             }
 
-            offset = nextOffset(offset);
+            offset = (offset << 2) + offset + perturb + 1;
+            perturb >>= PERTURB_SHIFT;
         }
     }
 
-    // linear search
-    int32_t nextOffset(int32_t offset) const {
-        offset += 1;
-        if (offset >= hash_table_size) {
-            offset = 0;
-        }
-        return offset;
-    }
-
     // add an item to the hash table
     void add(typed_python_hash_type hash, int32_t slot) {
         if (hash_table_count * 2 + 1 > hash_table_size
@@ -91,28 +86,29 @@ class hash_table_layout {
             resizeTable();
         }
 
-        // because typed_python has to use python's mod, which is
-        // different than c++'s mod for negative numbers, we just map
-        // negatives to positives.
         if (hash < 0) {
             hash = -hash;
         }
 
-        int32_t offset = hash % hash_table_size;
+        uint64_t mask = hash_table_size - 1;
+        uint64_t perturb = hash;
+        uint64_t offset = hash;
+
         while (true) {
-            if (hash_table_slots[offset] == EMPTY || hash_table_slots[offset] == DELETED) {
-                if (hash_table_slots[offset] == EMPTY) {
+            if (hash_table_slots[offset & mask] == EMPTY || hash_table_slots[offset & mask] == DELETED) {
+                if (hash_table_slots[offset & mask] == EMPTY) {
                     hash_table_empty_slots--;
                 }
 
-                hash_table_slots[offset] = slot;
-                hash_table_hashes[offset] = hash;
+                hash_table_slots[offset & mask] = slot;
+                hash_table_hashes[offset & mask] = hash;
                 items_populated[slot] = 1;
                 hash_table_count++;
                 return;
             }
 
-            offset = nextOffset(offset);
+            offset = (offset << 2) + offset + perturb + 1;
+            perturb >>= PERTURB_SHIFT;
         }
     }
 
@@ -134,17 +130,16 @@ class hash_table_layout {
             resizeTable();
         }
 
-        // because typed_python has to use python's mod, which is
-        // different than c++'s mod for negative numbers, we just map
-        // negatives to positives.
         if (hash < 0) {
             hash = -hash;
         }
 
-        int32_t offset = hash % hash_table_size;
+        uint64_t mask = hash_table_size - 1;
+        uint64_t perturb = hash;
+        uint64_t offset = hash;
 
         while (true) {
-            int32_t slot = hash_table_slots[offset];
+            int32_t slot = hash_table_slots[offset & mask];
 
             if (slot == EMPTY) {
                 // we never found the item
@@ -154,14 +149,15 @@ class hash_table_layout {
             if (slot != DELETED && compare(items + item_size * slot)) {
                 items_populated[slot] = 0;
 
-                hash_table_slots[offset] = DELETED;
-                hash_table_hashes[offset] = -1;
+                hash_table_slots[offset & mask] = DELETED;
+                hash_table_hashes[offset & mask] = -1;
                 hash_table_count -= 1;
 
                 return slot;
             }
 
-            offset = nextOffset(offset);
+            offset = (offset << 2) + offset + perturb + 1;
+            perturb >>= PERTURB_SHIFT;
         }
     }
 
@@ -187,9 +183,10 @@ class hash_table_layout {
             }
         }
 
+        items_populated = (uint8_t*)tp_realloc(items_populated, items_reserved, count_so_far);
+        items = (uint8_t*)tp_realloc(items, items_reserved * item_size, count_so_far * item_size);
+
         items_reserved = count_so_far;
-        items_populated = (uint8_t*)realloc(items_populated, count_so_far);
-        items = (uint8_t*)realloc(items, count_so_far * item_size);
         top_item_slot = items_reserved;
 
         for (long k = 0; k < hash_table_size; k++) {
@@ -218,9 +215,9 @@ class hash_table_layout {
     int32_t allocateNewSlot(size_t item_size) {
         if (!items) {
             items_reserved = 4;
-            items = (uint8_t*)malloc(items_reserved * item_size);
+            items = (uint8_t*)tp_malloc(items_reserved * item_size);
             std::memset(items, 0, items_reserved * item_size);
-            items_populated = (uint8_t*)malloc(items_reserved);
+            items_populated = (uint8_t*)tp_malloc(items_reserved);
             std::memset(items_populated, 0, items_reserved);
             top_item_slot = 0;
             for (long k = 0; k < items_reserved; k++) {
@@ -231,8 +228,8 @@ class hash_table_layout {
         while (top_item_slot >= items_reserved) {
             size_t old_reserved = items_reserved;
             items_reserved = items_reserved * 1.25 + 1;
-            items = (uint8_t*)realloc(items, item_size * items_reserved);
-            items_populated = (uint8_t*)realloc(items_populated, items_reserved);
+            items = (uint8_t*)tp_realloc(items, item_size * old_reserved, item_size * items_reserved);
+            items_populated = (uint8_t*)tp_realloc(items_populated, old_reserved, items_reserved);
 
             for (long k = old_reserved; k < items_reserved; k++) {
                 items_populated[k] = 0;
@@ -242,39 +239,12 @@ class hash_table_layout {
         return top_item_slot++;
     }
 
-    int32_t computeNextPrime(int32_t p) {
-        static std::vector<int32_t> primes;
-        if (!primes.size()) {
-            primes.push_back(2);
-        }
-
-        auto isprime = [&](int32_t candidate) {
-            for (auto d : primes) {
-                if (candidate % d == 0) {
-                    return false;
-                }
-                if (d * d > candidate) {
-                    return true;
-                }
-            }
-            throw std::logic_error("Expected to clear the primes list.");
-        };
-
-        while (true) {
-            while (primes.back() * primes.back() < p) {
-                int32_t cur = primes.back() + 1;
-                while (!isprime(cur)) {
-                    cur++;
-                }
-                primes.push_back(cur);
-            }
-
-            if (isprime(p)) {
-                return p;
-            }
-
-            p++;
+    int32_t pickHashTableSize(int32_t minSize) {
+        int32_t ct = MIN_SIZE;
+        while (ct < minSize) {
+            ct <<= 1;
         }
+        return ct;
     }
 
     // called after we have deleted everything that's populated, and need to
@@ -295,7 +265,7 @@ class hash_table_layout {
 
     template <class copy_constructor_type>
     hash_table_layout* copyTable(size_t item_size, bool isPOD, const copy_constructor_type& copy_constructor) {
-        hash_table_layout* result = (hash_table_layout*)malloc(sizeof(hash_table_layout));
+        hash_table_layout* result = (hash_table_layout*)tp_malloc(sizeof(hash_table_layout));
         new (result) hash_table_layout;
 
         result->refcount = 1;
@@ -308,7 +278,7 @@ class hash_table_layout {
         result->hash_table_count = hash_table_count;
         result->hash_table_empty_slots = hash_table_empty_slots;
 
-        result->items = (uint8_t*)malloc(item_size * items_reserved);
+        result->items = (uint8_t*)tp_malloc(item_size * items_reserved);
         if (isPOD) {
             memcpy(result->items, items, item_size * items_reserved);
         }
@@ -320,28 +290,23 @@ class hash_table_layout {
              }
         }
 
-        result->items_populated = (uint8_t*)malloc(items_reserved);
+        result->items_populated = (uint8_t*)tp_malloc(items_reserved);
         memcpy(result->items_populated, items_populated, items_reserved);
 
-        result->hash_table_slots = (int32_t*)malloc(hash_table_size * sizeof(int32_t));
+        result->hash_table_slots = (int32_t*)tp_malloc(hash_table_size * sizeof(int32_t));
         memcpy(result->hash_table_slots, hash_table_slots, hash_table_size * sizeof(int32_t));
 
-        result->hash_table_hashes = (typed_python_hash_type*)malloc(hash_table_size * sizeof(typed_python_hash_type));
+        result->hash_table_hashes = (typed_python_hash_type*)tp_malloc(hash_table_size * sizeof(typed_python_hash_type));
         memcpy(result->hash_table_hashes, hash_table_hashes, hash_table_size * sizeof(typed_python_hash_type));
         return result;
     }
 
     void resizeTable() {
         if (!hash_table_slots) {
-            // experiment with pre-allocation, but didn't detect performance improvement
-            if (!hash_table_count) {
-                hash_table_size = computeNextPrime(hash_table_count * 4 + 7);
-            }
-            else
-                hash_table_size = 7;
-            hash_table_slots = (int32_t*)malloc(hash_table_size * sizeof(int32_t));
+            hash_table_size = pickHashTableSize(hash_table_count * 4);
+            hash_table_slots = (int32_t*)tp_malloc(hash_table_size * sizeof(int32_t));
             setTo(hash_table_slots, EMPTY, hash_table_size);
-            hash_table_hashes = (typed_python_hash_type*)malloc(hash_table_size * sizeof(typed_python_hash_type));
+            hash_table_hashes = (typed_python_hash_type*)tp_malloc(hash_table_size * sizeof(typed_python_hash_type));
             setTo(hash_table_hashes, EMPTY, hash_table_size);
             hash_table_count = 0;
             hash_table_empty_slots = hash_table_size;
@@ -352,12 +317,12 @@ class hash_table_layout {
             typed_python_hash_type* oldHashes = hash_table_hashes;
 
             // make sure the table's not too small
-            hash_table_size = computeNextPrime(hash_table_count * 4 + 7);
+            hash_table_size = pickHashTableSize(hash_table_count * 4);
 
-            hash_table_slots = (int32_t*)malloc(hash_table_size * sizeof(int32_t));
+            hash_table_slots = (int32_t*)tp_malloc(hash_table_size * sizeof(int32_t));
             setTo(hash_table_slots, EMPTY, hash_table_size);
             hash_table_hashes =
-              (typed_python_hash_type*)malloc(hash_table_size * sizeof(typed_python_hash_type));
+              (typed_python_hash_type*)tp_malloc(hash_table_size * sizeof(typed_python_hash_type));
             setTo(hash_table_hashes, EMPTY, hash_table_size);
             hash_table_count = 0;
             hash_table_empty_slots = hash_table_size;
@@ -368,8 +333,8 @@ class hash_table_layout {
                 }
             }
 
-            free(oldSlots);
-            free(oldHashes);
+            tp_free(oldSlots);
+            tp_free(oldHashes);
         }
     }
 
@@ -380,8 +345,8 @@ class hash_table_layout {
         }
 
         items_reserved = slotCount;
-        items_populated = (uint8_t*)malloc(slotCount);
-        items = (uint8_t*)malloc(slotCount * item_size);
+        items_populated = (uint8_t*)tp_malloc(slotCount);
+        items = (uint8_t*)tp_malloc(slotCount * item_size);
 
         for (long k = 0; k < items_reserved; k++) {
             items_populated[k] = true;
@@ -392,10 +357,10 @@ class hash_table_layout {
 
     template <class hash_fun_type>
     void buildHashTableAfterDeserialization(size_t item_size, const hash_fun_type& hash_fun) {
-        hash_table_size = computeNextPrime(items_reserved * 2.5 + 7);
-        hash_table_slots = (int32_t*)malloc(hash_table_size * sizeof(int32_t));
+        hash_table_size = pickHashTableSize(items_reserved * 2);
+        hash_table_slots = (int32_t*)tp_malloc(hash_table_size * sizeof(int32_t));
         hash_table_hashes =
-          (typed_python_hash_type*)malloc(hash_table_size * sizeof(typed_python_hash_type));
+          (typed_python_hash_type*)tp_malloc(hash_table_size * sizeof(typed_python_hash_type));
         hash_table_count = 0;
         hash_table_empty_slots = hash_table_size;
 
@@ -409,6 +374,89 @@ class hash_table_layout {
         }
     }
 
+    hash_table_layout* deepcopy(
+        DeepcopyContext& context,
+        Type* dictOrSetType,
+        Type* keyType,
+        Type* valueType
+    ) {
+        hash_table_layout* dest = (hash_table_layout*)context.slab->allocate(sizeof(hash_table_layout), dictOrSetType);
+
+        new (dest) hash_table_layout();
+
+        dest->refcount = 1;
+
+        int bytesPerKVPair = keyType->bytecount() + (valueType ? valueType->bytecount() : 0);
+
+        dest->items = (uint8_t*)context.slab->allocate(
+            bytesPerKVPair * this->items_reserved,
+            nullptr
+        );
+
+        if (keyType->isPOD() && (!valueType || valueType->isPOD())) {
+            memcpy(
+                dest->items,
+                this->items,
+                this->items_reserved * bytesPerKVPair
+            );
+        } else {
+            for (long k = 0; k < this->items_reserved; k++) {
+                if (this->items_populated[k]) {
+                    keyType->deepcopy(
+                        dest->items + bytesPerKVPair * k,
+                        this->items + bytesPerKVPair * k,
+                        context
+                    );
+                    if (valueType) {
+                        valueType->deepcopy(
+                            dest->items + bytesPerKVPair * k + keyType->bytecount(),
+                            this->items + bytesPerKVPair * k + keyType->bytecount(),
+                            context
+                        );
+                    }
+                };
+            }
+        }
+
+        dest->items_populated = (uint8_t*)context.slab->allocate(
+            this->items_reserved,
+            nullptr
+        );
+        memcpy(
+            dest->items_populated,
+            this->items_populated,
+            this->items_reserved
+        );
+
+        dest->items_reserved = this->items_reserved;
+
+        dest->top_item_slot = this->top_item_slot;
+
+        dest->hash_table_count = this->hash_table_count;
+        dest->hash_table_size = this->hash_table_size;
+        dest->hash_table_empty_slots = this->hash_table_empty_slots;
+
+        dest->hash_table_slots = (int32_t*)context.slab->allocate(sizeof(int32_t) * this->hash_table_size, nullptr);
+        memcpy(
+            dest->hash_table_slots,
+            this->hash_table_slots,
+            sizeof(int32_t) * this->hash_table_size
+        );
+
+        dest->hash_table_hashes = (typed_python_hash_type*)context.slab->allocate(
+            sizeof(typed_python_hash_type) * this->hash_table_size,
+            nullptr
+        );
+        memcpy(
+            dest->hash_table_hashes,
+            this->hash_table_hashes,
+            sizeof(typed_python_hash_type) * this->hash_table_size
+        );
+
+        return dest;
+    }
+
+
     void checkInvariants(std::string reason) {
         int64_t popCount = 0;
         for (long k = 0; k < items_reserved; k++) {
@@ -485,3 +533,6 @@ class hash_table_layout {
                                    // table. Recall that some slots are
                                    // 'deleted'
 };
+
+
+typedef hash_table_layout* hash_table_layout_ptr;
