@@ -57,6 +57,10 @@ from typed_python._types import (
 module_level_testfun = dummy_test_module.testfunction
 
 
+def moduleLevelFunctionUsedByExactlyOneSerializationTest():
+    return "please don't touch me"
+
+
 def moduleLevelRecursiveF(x):
     if x > 0:
         return moduleLevelRecursiveF(x - 1) + 1
@@ -2411,6 +2415,19 @@ class TypesSerializationTest(unittest.TestCase):
 
         assert callF1(child1) == callF2(child2)
 
+    def test_identity_hash_of_lambda_doesnt_change_serialization(self):
+        s = SerializationContext()
+
+        f = lambda: 10
+
+        ser1 = s.serialize(f)
+
+        identityHash(f)
+
+        ser2 = s.serialize(f)
+
+        assert ser1 == ser2
+
     def test_call_method_dispatch_on_two_versions_of_same_class_with_recursion(self):
         Base = Forward("Base")
 
@@ -2655,3 +2672,96 @@ class TypesSerializationTest(unittest.TestCase):
         C = SerializationContext().deserialize(serializedC)
 
         assert C.anF() is C
+
+    def test_serialization_independent_of_whether_function_is_hashed(self):
+        s = SerializationContext().withoutLineInfoEncoded().withoutCompression()
+
+        s1 = s.serialize(moduleLevelFunctionUsedByExactlyOneSerializationTest)
+
+        identityHash(moduleLevelFunctionUsedByExactlyOneSerializationTest)
+
+        s2 = s.serialize(moduleLevelFunctionUsedByExactlyOneSerializationTest)
+
+        assert s1 == s2
+
+    def test_serialize_anonymous_class_with_defaults_and_nonempty(self):
+        class C1(Class):
+            x1 = Member(int, default_value=10, nonempty=True)
+            x2 = Member(str, default_value="10", nonempty=True)
+            x3 = Member(float, default_value=2.5)
+            x4 = Member(str, nonempty=True)
+
+        s = SerializationContext()
+
+        C2 = s.deserialize(s.serialize(C1))
+
+        assert C2.ClassMembers['x1'].type is int
+        assert C2.ClassMembers['x2'].type is str
+        assert C2.ClassMembers['x3'].type is float
+        assert C2.ClassMembers['x4'].type is str
+
+        assert C2.ClassMembers['x1'].isNonempty
+        assert C2.ClassMembers['x2'].isNonempty
+        assert not C2.ClassMembers['x3'].isNonempty
+        assert C2.ClassMembers['x4'].isNonempty
+
+        assert C2.ClassMembers['x1'].defaultValue == 10
+        assert C2.ClassMembers['x2'].defaultValue == "10"
+        assert C2.ClassMembers['x3'].defaultValue == 2.5
+        assert C2.ClassMembers['x4'].defaultValue is None
+
+    def test_serialize_unresolved_forward(self):
+        F = Forward("Hi")
+        T = NamedTuple(x=F)
+
+        s = SerializationContext()
+
+        T2 = s.deserialize(s.serialize(T))
+
+        assert T2.ElementTypes[0].__typed_python_category__ == "Forward"
+
+    def test_serialization_doesnt_starve_gil(self):
+        checkCount = [0]
+        serializeCount = [0]
+        contextSwaps = []
+        s = SerializationContext()
+
+        def loopThread():
+            t0 = time.time()
+            while time.time() - t0 < 1.0:
+                contextSwaps.append((0, time.time()))
+                checkCount[0] += 1
+
+        def serializeThread():
+            t0 = time.time()
+            while time.time() - t0 < 1.0:
+                contextSwaps.append((1, time.time()))
+                s.deserialize(s.serialize(moduleLevelRecursiveF))
+                serializeCount[0] += 1
+
+        threads = [threading.Thread(target=t) for t in [loopThread, serializeThread]]
+        for t in threads:
+            t.start()
+        for t in threads:
+            t.join()
+
+        contextSwaps = sorted(contextSwaps, key=lambda a: a[1])
+        contextSwaps = contextSwaps[:1] + [
+            contextSwaps[i] for i in range(1, len(contextSwaps))
+            if contextSwaps[i][0] != contextSwaps[i-1][0]
+        ]
+
+        avgTimeHeld = {0: 0.0, 1: 0.0}
+
+        for i in range(1, len(contextSwaps)):
+            avgTimeHeld[contextSwaps[i-1][0]] += contextSwaps[i][1] - contextSwaps[i-1][1]
+
+        print("Time spent in python spin thread:  ", avgTimeHeld[0], " over ", checkCount[0], " cycles")
+        print("Time spent in serialization thread:", avgTimeHeld[1], " over ", serializeCount[0], " cycles")
+
+        # verify that we are spending a reasonable amount of time in each thread. If
+        # we release the gil directly in 'deserialize' or 'serialize', then we'll end up
+        # starving the serialization thread because we only get the thread back every
+        # few milliseconds.
+        assert .1 <= avgTimeHeld[0] <= .9
+        assert .1 <= avgTimeHeld[1] <= .9
