@@ -23,6 +23,7 @@
 
 class Type;
 class Instance;
+class MemberDefinition;
 
 static_assert(PY_MAJOR_VERSION >= 3, "typed_python is a python3 project only");
 static_assert(PY_MINOR_VERSION >= 6, "typed_python is a python3.6 project only");
@@ -149,6 +150,10 @@ public:
         return m_ptr;
     }
 
+    PyObject* get() const {
+        return m_ptr;
+    }
+
     void set(PyObject* val) {
         if (val == m_ptr) {
             return;
@@ -201,7 +206,7 @@ bool unpackTupleToTypes(PyObject* tuple, std::vector<Type*>& out);
 
 bool unpackTupleToStringAndTypes(PyObject* tuple, std::vector<std::pair<std::string, Type*> >& out);
 
-bool unpackTupleToStringTypesAndValues(PyObject* tuple, std::vector<std::tuple<std::string, Type*, Instance> >& out);
+bool unpackTupleToMemberDefinition(PyObject* tuple, std::vector<MemberDefinition>& out);
 
 bool unpackTupleToStringAndObjects(PyObject* tuple, std::vector<std::pair<std::string, PyObject*> >& out);
 
